// last revised: 23:10 9/8/2010

function ModuleFormValidate() {
    var WSObj = new Object;

    WSObj.pattern = {
        'unsigned_integer' : new RegExp("^\+?[0-9]+$", "m"),
        'signed_integer' : new RegExp("^[\+\-]?[0-9]+$", "m")
    };
    WSObj.validateReplace = {
        'htmlspecialchars' : [['"', "'", '<', '>'], ['&quot;', '&#039', '&lt;', '&gt;']]
    };

    WSObj.validateSingle = function (value, setting) {
        if (typeof setting['batch'] != 'object' && typeof setting['batch'] != 'array') {
            var valueArray = [value];
        } else {
            var valueArray = value;
        }

        if (typeof setting['batch'] == 'object' || typeof setting['batch'] == 'array') {
            var count = WSObj.objLength(valueArray);
            if (count < setting['batch']['min']) {
                // if it is not enough
                throw "";
            } else if (setting['batch']['max'] != 'unlimited' && count > setting['batch']['max']) {
                // if it is too much
                throw "";
            }
        }

		for (var valueArrayKey in setting) {
			var valueArrayValue = setting[valueArrayKey];
			var singleValue = valueArray[valueArrayKey];

            switch (setting['type']) {
                case 'enum':
                    if (typeof setting['multipleSelect'] != 'undefined') {
                        var enumMultipleValue = singleValue;
                        var enumMultipleValueCount = WSObj.objLength(enumMultipleValue);

                        if (enumMultipleValueCount < setting['multipleSelect']['min']) {
                            throw "";
                        } else if (setting['multipleSelect']['max'] != 'unlimited' && enumMultipleValueCount > setting['multipleSelect']['max']) {
                            throw "";
                        }

                        for (var key in enumMultipleValue) {
                            var valueToCheck = enumMultipleValue[key];
                            if (!WSObj.in_array(valueToCheck, setting['value'])) {
                            	throw "";
                            }
                        }
                    } else {
                        if (!WSObj.in_array(singleValue, setting['value'])) {
                        	throw "";
                        }
                    }
                    break;
                case 'integer':
                    if (setting['unsigned']) {
                        if (!singleValue.match(WSObj.pattern['unsigned_integer'])) {
                            throw "";
                        }
                        singleValue = Math.abs(parseInt(singleValue));
                    } else {
                        if (!singleValue.match(WSObj.pattern['signed_integer'])) {
                            throw "";
                        }
                        singleValue = parseInt(singleValue);
                    }

                    if (setting['lower'] != 'unlimited' && singleValue < setting['lower']) {
                        throw "";
                    } else if (setting['upper'] != 'unlimited' && singleValue > setting['upper']) {
                        throw "";
                    }
                    break;
                case 'string':
                    var length = WSObj.objLength(singleValue);

                    if (length < setting['length']['min']) {
                        throw "";
                    } else if (setting['length']['max'] != 'unlimited' && length > setting['length']['max']) {
                        if (typeof setting['length']['onExceedMax'] != 'undefined' && setting['length']['onExceedMax'] == 'cut') {
                            singleValue = singleValue.substr(0, setting['length']['max']);
                        } else {
                            throw "";
                        }
                    }

                    if (typeof setting['replace'] == 'object' || typeof setting['replace'] == 'array') {
                        for (var key in setting['replace']) {
                            if (typeof singleValue.search(setting['replace'][key]) == string) {
                            	for (var replaceKey in WSObj.validateReplace[setting['replace'][key]][0]) {
	                                while (singleValue.search(WSObj.validateReplace[setting['replace'][key]][0][replaceKey]) != -1) {
	                                    singleValue.replace(WSObj.validateReplace[setting['replace'][key]][0][replaceKey], WSObj.validateReplace[setting['replace'][key]][1][replaceKey]);
	                                }
                                }
                            } else {
                            	for (var replaceKey in setting['replace'][key][0]) {
	                                while (singleValue.search(setting['replace'][key][0][replaceKey]) != -1) {
	                                    singleValue.replace(setting['replace'][key][0][replaceKey], setting['replace'][key][1][replaceKey]);
	                                }
                                }
                            }
                        }
                    }

                    if ((typeof setting['pregMatch'] == 'object' || typeof setting['pregMatch'] == 'array') && WSObj.objLength(setting['pregMatch']) > 0) {
                        var matched = false;

                        for (var pregMatchKey in setting['pregMatch']) {
                            var pregMatchDataSet = setting['pregMatch'][pregMatchKey];

                            var thisTimeMatched = true;

                            for (var pregMatchDataSetKey in pregMatchDataSet) {
                            	var pregMatchData = pregMatchDataSet[pregMatchDataSetKey];
                                if (!singleValue.match(pregMatchData[0])) {
                                    thisTimeMatched = false;
                                    break;
                                }
                            }

                            if (thisTimeMatched) {
                                matched = true;
                                break;
                            }
                        }

                        if (!matched) {
                            throw "";
                        }
                    }
                    break;
            }

            valueArray[valueArrayKey] = singleValue;
		}

        return (typeof setting['batch'] != 'object' && typeof setting['batch'] != 'array') ? valueArray[0] : valueArray;
    }

    WSObj.validate = function (dataToValidate, setting) {
    	var cacheExceptionWithCondition = {};
        var validatedList = {};

        for (var name in setting) {
            var data = setting[name];

            if (typeof dataToValidate[name] != 'undefined') {
                try {
                    WSObj.validateSingle(dataToValidate[name], data);
                } catch (e) {
                	if (data['mustExist']) {
                		throw "";
	                } else {
	                    if (data['onValidationFail'] == 'useDefault') {
	                        dataToValidate[name] = data['default'];
	                    } else if (data['onValidationFail'] == 'unset') {
	                        delete dataToValidate[name];
	                    } else {
	                        delete dataToValidate[name];
	                    }
	                }
                }
            } else if (data['mustExist']) {
            	throw "";
            }

            validatedList[name] = true;
        }

        for (var name in dataToValidate) {
        	var data = dataToValidate[name];
            if (typeof validatedList[name] == 'undefined') {
                delete dataToValidate[name];
            }
        }

		try {
			WSObj.checkAllConditions(dataToValidate, setting, cacheExceptionWithCondition);
		} catch (e) {
			throw "";
		}

		for (var name in cacheExceptionWithCondition) {
			var exception = cacheExceptionWithCondition[name];
			throw "";
		}

        return true;
    }

	WSObj.checkAllConditions = function (dataToValidate, setting, cacheExceptionWithCondition) {
		while (true) {
			var unsetExist = false;

			for (var name in setting) {
				var data = setting[name];

				if ((typeof data['condition'] != 'array' && typeof data['condition'] != 'object') || typeof dataToValidate[name] == 'undefined') {
					continue;
				}

				var anyConditionSetMatched = WSObj.checkSingleCondition(dataToValidate, setting, name, data['condition']);

				if (!anyConditionSetMatched) {
					if (data['onConditionFail'] == 'useDefault') {
						dataToValidate[name] = data['default'];
					} else if (data['onConditionFail'] == 'unset') {
						delete dataToValidate[name];
						unsetExist = true;
					} else {
						delete dataToValidate[name];
						unsetExist = true;
					}
				} else {
					if (typeof cacheExceptionWithCondition[name] != 'undefined') {
						if (data['mustExist']) {
							throw "";
						} else {
							if (data['onValidationFail'] == 'useDafault') {
								dataToValidate[name] = data['default'];
							} else if (data['onValidationFail'] == 'unset') {
								delete dataToValidate[name];
								unsetExist = true;
							} else {
								delete dataToValidate[name];
								unsetExist = true;
							}

							delete cacheExceptionWithCondition[name];
						}
					}
				}
			}

			if (!unsetExist) {
				break;
			}
		}
	}

    WSObj.checkSingleCondition = function (dataToValidate, setting, elementName, conditions) {
        var mainMatch = false;

        for (var key in conditions) {
        	var conditionSet = conditions[key];
            var subMatch = true;

            for (var conditionVarName in conditionSet) {
            	var conditionValue = conditionSet[conditionVarName];
                var targetName = conditionVarName.substr(0, 1) == ':' ? WSObj.extractName(conditionVarName) : conditionVarName;

                if (typeof dataToValidate[targetName] == 'undefined') {
                    subMatch = false;
                    break;
                }

                if (typeof setting[targetName]['batch'] != 'undefined') {
                    if (!WSObj.checkSingleConditionBatch(dataToValidate, conditionVarName, conditionValue, elementName, typeof setting[elementName]['batch'] != 'undefined')) {
                        subMatch = false;
                        break;
                    }
                } else {
                    if (conditionValue != 'matched') {
                        if (setting[targetName]['type'] == 'enum' && typeof setting[targetName]['multipleSelect'] != 'undefined') {

                            var diffCount = WSObj.objLength(WSObj.array_diff(conditionValue['value'], dataToValidate[targetName]));
                            if (conditionValue['mode'] == 'fit') {
                                if (diffCount != 0) {
                                    subMatch = false;
                                    break;
                                }
                            } else if (conditionValue['mode'] == 'allWithin') {
                                var conditionCount = WSObj.objLength(conditionValue['value']);
                                var targetCount = WSObj.objLength(dataToValidate[targetName]);
                                if (targetCount > conditionCount || conditionCount - targetCount != diffCount) {
                                    subMatch = false;
                                    break;
                                }
                            } else if (conditionValue['mode'] == 'any') {
                                var conditionCount = WSObj.objLength(conditionValue['value']);
                                var targetCount = WSObj.objLength(dataToValidate[targetName]);
                                if (diffCount == conditionCount + targetCount) {
                                    subMatch = false;
                                    break;
                                }
                            } else {
                                subMatch = false;
                                break;
                            }

                        } else if (!WSObj.in_array(dataToValidate[targetName], conditionValue)) {
                            subMatch = false;
                            break;
                        }
                    }
                }
            }

            if (subMatch) {
                mainMatch = true;
                break;
            }
        }

        return mainMatch;
    }

    WSObj.checkSingleConditionBatch = function (dataToValidate, conditionVarName, conditionValue, elementName, elementIsBatch) {
        var targetName = WSObj.extractName(conditionVarName);

        if (elementIsBatch) {
            var count = 0;
            for (var dataToValidateKey in dataToValidate[elementName]) {
            	var myData = dataToValidate[elementName][dataToValidateKey]

                var rangeArray = WSObj.extractRange(conditionVarName, WSObj.objLength(dataToValidate[targetName]), count);

                var count2 = 0;
                for (var key in dataToValidate[targetName]) {
                	var targetData = dataToValidate[targetName][key];
                    if (typeof rangeArray[count2] != 'undefined') {
                        if (conditionValue != 'matched' && !WSObj.in_array(targetData, conditionValue)) {
                            return false;
                        }
                        delete rangeArray[count2];
                    }

                    count2++;
                }

                if (WSObj.objLength(rangeArray) != 0) {
                    return false;
                }

                count++;
            }
        } else {
            rangeArray = WSObj.extractRange(conditionVarName, WSObj.objLength(dataToValidate[targetName]), false);

            count = 0;
            for (var key in dataToValidate[targetName]) {
            	var targetData = dataToValidate[targetName][key];
                if (typeof rangeArray[count] != 'undefined') {
                    if (conditionValue != 'matched' && !WSObj.in_array(targetData, conditionValue)) {
                        return false;
                    }
                    delete rangeArray[count];
                }

                count++;
            }

            if (WSObj.objLength(rangeArray) != 0) {
                return false;
            }
        }

        return true;
    }

    WSObj.getDefaultValue = function (setting) {
        var output = {};
        for (var name in setting) {
            var data = setting[name];

			output[name] = {};
            output[name]['batch'] = typeof setting['batch'] != 'undefined';

            if (setting['type'] == 'enum') {
            	var elementTypeChecked = WSObj.in_array(setting['elementType'], ['checkbox', 'radio']) ? 'checked="checked"' : 'selected="selected"';

            	if (typeof setting['multipleSelect'] != 'undefined') {
                	if (output[name]['batch']) {
                		for (var setKey in data['default']) {
                			var valueSet = data['default'][setKey];
                			for (var key in valueSet) {
                				var value = valueSet[key];
								output[name]['form'][setKey][value] = elementTypeChecked;
							}
                		}
                	} else {
                		for (var key in data['default']) {
                			var value = data['default'][key];
                			output[name]['form'][value] = elementTypeChecked;
                		}
                	}
                } else {
                	if (output[name]['batch']) {
                		for (var setKey in data['default']) {
                			var value = data['default'][setKey];
							output[name]['form'][setKey][value] = elementTypeChecked;
                		}
                	} else {
                		output[name]['form'][data['default']] = elementTypeChecked;
                	}
                }
            } else {
                output[name]['form'] = data['default'];
          	}
        }
        return output;
    }

    WSObj.mergeDefaultValue = function (defaultValue) {
    	var userInput = {};
    	for (var name in defaultValue) {
    		var data = defaultValue[name];
    		if (data['useDefault']) {
    			userInput[name] = data['default'];
    		}
    	}
    	return userInput;
    }

	WSObj.extractRange = function (input, targetNum, currentPosition) {
		if (typeof currentPosition == 'undefined') {
			currentPosition = false;
		}

		var resultArray = {};

		var treatInput = input.substr(1).split(':');
		treatInput = treatInput[0].split(',');
		for (var key in treatInput) {
			var range = treatInput[key];

			range = range.split('-');
			try {
				var start = WSObj.decodeRangeCode(range[0], targetNum, currentPosition);
				var end = WSObj.decodeRangeCode(range[1], targetNum, currentPosition);
				for (var i = start; i <= end; i++) {
					resultArray[i] = true;
				}
			} catch (e) {
				// 1 if notation is invalid
			}
		}

		return resultArray;
	}

	WSObj.decodeRangeCode = function (keyword, targetNum, currentPosition) {
		switch (keyword[0]) {
			case 's':
				return 0;
			case 'e':
				return (targetNum - 1);
			case 'b':
				if (currentPosition === false) {
					throw "";
				}
				return Math.max(0, currentPosition - parseInt(keyword.substr(1)));
			case 'a':
				if (currentPosition === false) {
					throw "";
				}
				return Math.min(currentPosition + parseInt(keyword.substr(1)), targetNum - 1);
			case 'c':
				if (currentPosition === false) {
					throw "";
				}
				return currentPosition;
			default:
				return parseInt(keyword);
		}
	}

    WSObj.extractSetting = function (setting, nameArray) {
    	var output = {};
    	for (var name in setting) {
    		var data = setting[name];
    		if (WSObj.in_array(name, nameArray)) {
    			output[name] = data;
    		}
    	}
    	return output;
    }

	WSObj.objLength = function (obj) {
		var count = 0;
		for (var i in obj) {
			count++;
		}
		return count;
	}

	WSObj.in_array = function (toSearch, targetArray) {
		for (var i in targetArray) {
			if (toSearch == targetArray[i]) {
				return true;
			}
		}
		return false;
	}

	WSObj.array_diff = function (array1, array2) {
		var arrayCount1 = WSObj.objLength(array1);
		var arrayCount2 = WSObj.objLength(array2);
		var total = arrayCount1 + arrayCount2;

		var arrayWithMoreElements = arrayCount1 >= arrayCount2 ? array1 : array2;
		var arrayWithLessElement = arrayCount1 >= arrayCount2 ? array2 : array1;

		for (var i in arrayWithMoreElements) {
			var result = WSObj.inArray(arrayWithMoreElements[i], arrayWithLessElements);
			if (result) {
				total -= 2;
				delete arrayWithLessElements[result];
				delete EarrayWithMoreElements[i];
			}
		}

		return total;
	}

	WSObj.inArray = function (toSearch, targetArray) {
		for (var i in targetArray) {
			if (toSearch == targetArray[i]) {
				return i;
			}
		}
		return false;
	}

    return WSObj;
}