/*
    http://www.JSON.org/json2.js
    2010-03-20

    Public Domain.

    This file creates a global JSON object containing two methods: stringify
    and parse.

    JSON.stringify(value, replacer, space)
        value       any JavaScript value, usually an object or array.

        replacer    an optional parameter that determines how object
                    values are stringified for objects. It can be a
                    function or an array of strings.

        space       an optional parameter that specifies the indentation
                    of nested structures. If it is omitted, the text will
                    be packed without extra whitespace. If it is a number,
                    it will specify the number of spaces to indent at each
                    level. If it is a string (such as '\t' or '&nbsp;'),
                    it contains the characters used to indent at each level.

        This method produces a JSON text from a JavaScript value.

        When an object value is found, if the object contains a toJSON
        method, its toJSON method will be called and the result will be
        stringified. A toJSON method does not serialize: it returns the
        value represented by the name/value pair that should be serialized,
        or undefined if nothing should be serialized. The toJSON method
        will be passed the key associated with the value, and this will be
        bound to the value

        For example, this would serialize Dates as ISO strings.

            Date.prototype.toJSON = function (key) {
                function f(n) {
                    // Format integers to have at least two digits.
                    return n < 10 ? '0' + n : n;
                }

                return this.getUTCFullYear()   + '-' +
                     f(this.getUTCMonth() + 1) + '-' +
                     f(this.getUTCDate())      + 'T' +
                     f(this.getUTCHours())     + ':' +
                     f(this.getUTCMinutes())   + ':' +
                     f(this.getUTCSeconds())   + 'Z';
            };

        You can provide an optional replacer method. It will be passed the
        key and value of each member, with this bound to the containing
        object. The value that is returned from your method will be
        serialized. If your method returns undefined, then the member will
        be excluded from the serialization.

        If the replacer parameter is an array of strings, then it will be
        used to select the members to be serialized. It filters the results
        such that only members with keys listed in the replacer array are
        stringified.

        Values that do not have JSON representations, such as undefined or
        functions, will not be serialized. Such values in objects will be
        dropped; in arrays they will be replaced with null. You can use
        a replacer function to replace those with JSON values.
        JSON.stringify(undefined) returns undefined.

        The optional space parameter produces a stringification of the
        value that is filled with line breaks and indentation to make it
        easier to read.

        If the space parameter is a non-empty string, then that string will
        be used for indentation. If the space parameter is a number, then
        the indentation will be that many spaces.

        Example:

        text = JSON.stringify(['e', {pluribus: 'unum'}]);
        // text is '["e",{"pluribus":"unum"}]'


        text = JSON.stringify(['e', {pluribus: 'unum'}], null, '\t');
        // text is '[\n\t"e",\n\t{\n\t\t"pluribus": "unum"\n\t}\n]'

        text = JSON.stringify([new Date()], function (key, value) {
            return this[key] instanceof Date ?
                'Date(' + this[key] + ')' : value;
        });
        // text is '["Date(---current time---)"]'


    JSON.parse(text, reviver)
        This method parses a JSON text to produce an object or array.
        It can throw a SyntaxError exception.

        The optional reviver parameter is a function that can filter and
        transform the results. It receives each of the keys and values,
        and its return value is used instead of the original value.
        If it returns what it received, then the structure is not modified.
        If it returns undefined then the member is deleted.

        Example:

        // Parse the text. Values that look like ISO date strings will
        // be converted to Date objects.

        myData = JSON.parse(text, function (key, value) {
            var a;
            if (typeof value === 'string') {
                a = /^(\d{4})-(\d{2})-(\d{2})T(\d{2}):(\d{2}):(\d{2}(?:\.\d*)?)Z$/.exec(value);
                if (a) {
                    return new Date(Date.UTC(+a[1], +a[2] - 1, +a[3], +a[4],
                        +a[5], +a[6]));
                }
            }
            return value;
        });

        myData = JSON.parse('["Date(09/09/2001)"]', function (key, value) {
            var d;
            if (typeof value === 'string' &&
                    value.slice(0, 5) === 'Date(' &&
                    value.slice(-1) === ')') {
                d = new Date(value.slice(5, -1));
                if (d) {
                    return d;
                }
            }
            return value;
        });
*/

if(!this.JSON){this.JSON={}}(function(){function f(n){return n<10?"0"+n:n}if(typeof Date.prototype.toJSON!=="function"){Date.prototype.toJSON=function(key){return isFinite(this.valueOf())?this.getUTCFullYear()+"-"+f(this.getUTCMonth()+1)+"-"+f(this.getUTCDate())+"T"+f(this.getUTCHours())+":"+f(this.getUTCMinutes())+":"+f(this.getUTCSeconds())+"Z":null};String.prototype.toJSON=Number.prototype.toJSON=Boolean.prototype.toJSON=function(key){return this.valueOf()}}var cx=/[\u0000\u00ad\u0600-\u0604\u070f\u17b4\u17b5\u200c-\u200f\u2028-\u202f\u2060-\u206f\ufeff\ufff0-\uffff]/g,escapable=/[\\\"\x00-\x1f\x7f-\x9f\u00ad\u0600-\u0604\u070f\u17b4\u17b5\u200c-\u200f\u2028-\u202f\u2060-\u206f\ufeff\ufff0-\uffff]/g,gap,indent,meta={"\b":"\\b","\t":"\\t","\n":"\\n","\f":"\\f","\r":"\\r",'"':'\\"',"\\":"\\\\"},rep;
function quote(string){escapable.lastIndex=0;return escapable.test(string)?'"'+string.replace(escapable,function(a){var c=meta[a];return typeof c==="string"?c:"\\u"+("0000"+a.charCodeAt(0).toString(16)).slice(-4)})+'"':'"'+string+'"'}function str(key,holder){var i,k,v,length,mind=gap,partial,value=holder[key];if(value&&typeof value==="object"&&typeof value.toJSON==="function"){value=value.toJSON(key)}if(typeof rep==="function"){value=rep.call(holder,key,value)}switch(typeof value){case"string":return quote(value);
case"number":return isFinite(value)?String(value):"null";case"boolean":case"null":return String(value);case"object":if(!value){return"null"}gap+=indent;partial=[];if(Object.prototype.toString.apply(value)==="[object Array]"){length=value.length;for(i=0;i<length;i+=1){partial[i]=str(i,value)||"null"}v=partial.length===0?"[]":gap?"[\n"+gap+partial.join(",\n"+gap)+"\n"+mind+"]":"["+partial.join(",")+"]";gap=mind;return v}if(rep&&typeof rep==="object"){length=rep.length;for(i=0;i<length;i+=1){k=rep[i];
if(typeof k==="string"){v=str(k,value);if(v){partial.push(quote(k)+(gap?": ":":")+v)}}}}else{for(k in value){if(Object.hasOwnProperty.call(value,k)){v=str(k,value);if(v){partial.push(quote(k)+(gap?": ":":")+v)}}}}v=partial.length===0?"{}":gap?"{\n"+gap+partial.join(",\n"+gap)+"\n"+mind+"}":"{"+partial.join(",")+"}";gap=mind;return v}}if(typeof JSON.stringify!=="function"){JSON.stringify=function(value,replacer,space){var i;gap="";indent="";if(typeof space==="number"){for(i=0;i<space;i+=1){indent+=" "
}}else{if(typeof space==="string"){indent=space}}rep=replacer;if(replacer&&typeof replacer!=="function"&&(typeof replacer!=="object"||typeof replacer.length!=="number")){throw new Error("JSON.stringify")}return str("",{"":value})}}if(typeof JSON.parse!=="function"){JSON.parse=function(text,reviver){var j;function walk(holder,key){var k,v,value=holder[key];if(value&&typeof value==="object"){for(k in value){if(Object.hasOwnProperty.call(value,k)){v=walk(value,k);if(v!==undefined){value[k]=v}else{delete value[k]
}}}}return reviver.call(holder,key,value)}text=String(text);cx.lastIndex=0;if(cx.test(text)){text=text.replace(cx,function(a){return"\\u"+("0000"+a.charCodeAt(0).toString(16)).slice(-4)})}if(/^[\],:{}\s]*$/.test(text.replace(/\\(?:["\\\/bfnrt]|u[0-9a-fA-F]{4})/g,"@").replace(/"[^"\\\n\r]*"|true|false|null|-?\d+(?:\.\d*)?(?:[eE][+\-]?\d+)?/g,"]").replace(/(?:^|:|,)(?:\s*\[)+/g,""))){j=eval("("+text+")");return typeof reviver==="function"?walk({"":j},""):j}throw new SyntaxError("JSON.parse")}}}());

// last revised: 14:08 5/8/2010
// tested on fx3, fx3.5, fx3.6, ie6, ie7, ie8, chrome5, opera10, safari5, XHTML 1.0 Transitional

/**
 * This class is used to perform easy AJAX data request (HTTP).
 * This class include both GET and POST method.
 * This class support response data be HTML or XML.
 */
function ModuleAjax() {

	var WSObj = new Object();

	/**
	 * private array recvType, child element is either 'HTML' or 'XML'
	 * private array resultHandle, child element is function
	 * private array XMLHttpRequest, child element is AJAX HTTP request/connection handler
	 */
	WSObj.recvType = new Array();
	WSObj.resultHandle = new Array();
	WSObj.XMLHttpRequest = new Array();

	/**
	 * Private. Prepare a AJAX connection, no URL is involved so far
	 * @return Return the request handler.
	 */
	WSObj.createXMLHttpRequest = function() {
		var request;
		
		if (window.XMLHttpRequest) {
			request = new XMLHttpRequest();
		}
		else if (window.ActiveXObject) {
			try {
				request = new ActiveXObject('Microsoft.XMLHTTP');
			} catch (e) {
				request = false;
			}
		}

		if (request == null || !request) alert('藍_, A{bϥΪs䴩 AJAX. Your browser do not support AJAX request.');

		return request;
	}

	/**
	 * Private. After we have got the response from AJAX request, we call this function to handle the rest(final) of the works.
	 * @param integer diffhash the unique identifier of each connection
	 * @return Nothing return.
	 */
	WSObj.processHandle = function(diffhash) {
		if (WSObj.XMLHttpRequest[diffhash] == null)
			alert('Warnning XMLHttpRequest['+diffhash+'] is null');
			
		if (WSObj.XMLHttpRequest[diffhash].readyState == 4 || WSObj.XMLHttpRequest[diffhash].readyState=="complete") {
			if (WSObj.XMLHttpRequest[diffhash].status == 200) {
				if (WSObj.recvType[diffhash] == 'HTML') {
					WSObj.resultHandle[diffhash](WSObj.XMLHttpRequest[diffhash].responseText);
				} else if (WSObj.recvType[diffhash] == 'XML') {
					WSObj.resultHandle[diffhash](WSObj.XMLHttpRequest[diffhash].responseXML.lastChild.firstChild.nodeValue);
				}
			} else {
				alert('AJAX: Page not found.');
			}
			WSObj.deleteRequest(diffhash);
		}
	}

	/**
	 * @param string recvType either 'HTML' or 'XML'
	 * @param string targetUrl the complete URL of the destination, should include protocal type, e.g. 'http://'
	 * @param function resultHandle after the AJAX request is completed, the obtained data will be passed into the resultHandle(obtained_data), e.g. function(content) { alert(content); }
	 * @return the identifier of the AJAX request
	 */
	WSObj.get = function(recvType, targetUrl, resultHandle) {
		// var diffhash = (new Date()).getTime();
		var diffhash = Math.random();

		WSObj.recvType[diffhash] = recvType == 'HTML' ? 'HTML' : 'XML';

		WSObj.XMLHttpRequest[diffhash] = WSObj.createXMLHttpRequest();
		WSObj.resultHandle[diffhash] = resultHandle;
		WSObj.XMLHttpRequest[diffhash].onreadystatechange = function () { WSObj.processHandle(diffhash) };

		WSObj.XMLHttpRequest[diffhash].open('GET', targetUrl + (targetUrl.match(/\?/) == '?' ? '&' : '?') + 'diffhash=' + diffhash, true);
		WSObj.XMLHttpRequest[diffhash].send(null);

		return diffhash;
	}

	/**
	 * @param string recvType either 'HTML' or 'XML'
	 * @param string targetUrl the complete URL of the destination, should include protocal type, e.g. 'http://'
	 * @param string sendString data that want to send by POST, no '?' at the beginning, use '&' instead of '&amp;', e.g. var1=1&var2=2&var3=3
	 * @param function resultHandle after the AJAX request is completed, the obtained data will be passed into the resultHandle(obtained_data), e.g. function(content) { alert(content); }
	 * @return the identifier of the AJAX request
	 */
	WSObj.post = function(recvType, targetUrl, sendString, resultHandle) {
		var diffhash = (new Date()).getTime();
		sendString = sendString + (sendString ? '&' : '') + 'diffhash=' + diffhash;

		WSObj.recvType[diffhash] = recvType == 'HTML' ? 'HTML' : 'XML';

		WSObj.XMLHttpRequest[diffhash] = WSObj.createXMLHttpRequest();
		WSObj.resultHandle[diffhash] = resultHandle;
		WSObj.XMLHttpRequest[diffhash].onreadystatechange = function () { WSObj.processHandle(diffhash) };

		WSObj.XMLHttpRequest[diffhash].open('POST', targetUrl, true);
		WSObj.XMLHttpRequest[diffhash].setRequestHeader('Content-Type', 'application/x-www-form-urlencoded');
		WSObj.XMLHttpRequest[diffhash].send(sendString);

		return diffhash;
	}

	/**
	 * @return integer 1 if succeed, -1 if browser don't support abort action, -2 if that AJAX request not exist
	 */
	WSObj.abort = function(label) {
		if (typeof WSObj.XMLHttpRequest[label] != 'undefined' && WSObj.XMLHttpRequest[label] != null) {
			// IE 6 or below don't support abort
			if (typeof WSObj.XMLHttpRequest[label].abort != 'undefined' || WSObj.XMLHttpRequest[label].abort != null) {
				return -1;
			}
			WSObj.XMLHttpRequest[label].abort();
			WSObj.deleteRequest(label);
			return 1;
		}
		return -2;
	}

	WSObj.deleteRequest = function (label) {
		delete WSObj.recvType[label];
		delete WSObj.resultHandle[label];
		delete WSObj.XMLHttpRequest[label];
	}

	return WSObj;
}

// last revised: 12:25 9/8/2010
// only limited on vertical expand and collapse
// tested on fx3, fx3.5, fx3.6, ie6, ie7, ie8, ie9b4, chrome5, opera10, safari5, XHTML 1.0 Transitional

function ModuleCollapseExpand() {
	var WSObj = new Object;

    //var userAgent = navigator.userAgent.toLowerCase();
    //WSObj.isIE = window.ActiveXObject && userAgent.indexOf('msie') != -1 && userAgent.substr(userAgent.indexOf('msie') + 5, 3);

	WSObj.speed = WSObj.IE ? 1.3 : 1.1;
	WSObj.initSpeed = 0.8;
	WSObj.interval = 15;	// in 1/1000 second

	WSObj.objOriginalDimension = new Object();

	WSObj.move = new Object();
	WSObj.callBackFunc = new Object();

	WSObj.initCollapsed = function (obj) {
		obj = WSObj.$(obj);
		var objId = obj.id;

		if (typeof WSObj.objOriginalDimension[objId] == 'undefined') {
			WSObj.objOriginalDimension[objId] = [obj.offsetWidth, obj.offsetHeight];
		}

		obj.style.overflow = 'hidden';
		obj.style.height = '0px';
	}
	
	// added by chris
	WSObj.UpdateCollapsed = function (obj) {
		obj = WSObj.$(obj);
		var objId = obj.id;

		WSObj.objOriginalDimension[objId] = [obj.offsetWidth, obj.offsetHeight];

		// obj.style.overflow = 'hidden';
		// obj.style.height = '0px';
	}

	WSObj.perform = function (type, obj, callbackFunc) {
		obj = WSObj.$(obj);
		var objId = obj.id;

		WSObj.stop(objId);

		WSObj.callBackFunc[objId] = typeof callbackFunc == 'undefined' ? function() { } : callbackFunc;

		if (typeof WSObj.objOriginalDimension[objId] == 'undefined') {
			WSObj.objOriginalDimension[objId] = [obj.offsetWidth, obj.offsetHeight];
		}

		obj.style.height = obj.offsetHeight + 'px';

		if (type == 'collapse') {
			WSObj.move[objId] = WSObj.getStep(obj.offsetHeight, 0, WSObj.speed, WSObj.initSpeed);
		} else {
			WSObj.move[objId] = WSObj.getStep(obj.offsetHeight, WSObj.objOriginalDimension[objId][1], WSObj.speed, WSObj.initSpeed);
		}

		obj.style.overflow = 'hidden';

		if (!WSObj.moveStarted) {
			WSObj.start();
		}
	}

	WSObj.moveStarted = false;
	WSObj.intervalHandler = false;
	WSObj.start = function () {
		WSObj.moveStarted = true;

		var i, j;
		for (i in WSObj.callBackFunc) {
			var obj = WSObj.$(i);
			for (j in WSObj.move[i]) {
				obj.style.height = (parseInt(obj.style.height) + WSObj.move[i][j]) + 'px';
				delete WSObj.move[i][j];
				if (j == WSObj.move[i].length - 1) {
					WSObj.stop(i);
				}
				break;
			}
		}

		var objListIsEmpty = true;
		for (i in WSObj.callBackFunc) {
			objListIsEmpty = false;
			break;
		}
		if (objListIsEmpty) {
			clearInterval(WSObj.intervalHandler);
			WSObj.intervalHandler = false;
			WSObj.moveStarted = false;
		} else if (!WSObj.intervalHandler) {
			// of course objListIsEmpty is false
			WSObj.intervalHandler = setInterval(WSObj.start, WSObj.interval);
		}
	}

	WSObj.stop = function (objId) {
		if (typeof WSObj.callBackFunc[objId] != 'undefined') WSObj.callBackFunc[objId]();
		delete WSObj.move[objId];
		delete WSObj.callBackFunc[objId];
	}

	WSObj.getStep = function (start, stop, speed, initSpeed) {
		initSpeed = typeof initSpeed != 'undefined' ? initSpeed : 1;

		var sign = start - stop;
		var distance = Math.abs(sign);
		var currentSpeed = initSpeed;

		var tempResult = [];
		var proceeded = 0, stepSize;
		while (proceeded < distance) {
			currentSpeed = currentSpeed * speed;
			stepSize = Math.ceil(Math.exp(currentSpeed));
			proceeded += stepSize;
			if (proceeded > distance) {
				stepSize -= (proceeded - distance);
				proceeded = distance;
			}
			tempResult[tempResult.length] = stepSize;
		}

		var returnResult = [];
		for (var i = tempResult.length - 1; i >= 0; i--) {
			returnResult[returnResult.length] = sign > 0 ? -tempResult[i] : tempResult[i];
		}

		return returnResult;
	}

	/**
	 * Protected
	 */
    WSObj.$ = function (obj) {
        return typeof obj == 'string' ? document.getElementById(obj) : obj;
    }

	return WSObj;
}

// last revised: 12:26 9/8/2010
// only limited on vertical movement
// tested on fx3, fx3.5, fx3.6, ie6, ie7, ie8, ie9b4, chrome5, opera10, safari5, XHTML 1.0 Transitional

function ModuleElementMove() {
	var WSObj = new Object();

    var userAgent = navigator.userAgent.toLowerCase();
    WSObj.isIE = window.ActiveXObject && userAgent.indexOf('msie') != -1 && userAgent.substr(userAgent.indexOf('msie') + 5, 3);

	WSObj.speed = WSObj.IE ? 1.3 : 1.1;
	WSObj.initSpeed = 0.8;
	WSObj.interval = 15;

	WSObj.objCallbackFunc = new Object();
	WSObj.moveMove = new Object();
	WSObj.move = function (objId, callbackFunc, toY) {
		var obj = WSObj.$(objId);

		WSObj.objCallbackFunc[objId] = callbackFunc;
		WSObj.moveMove[objId] = WSObj.getStep(obj.offsetTop, toY, WSObj.speed, WSObj.initSpeed);

		if (!WSObj.moveStarted) {
			WSObj.moveStart();
		}
	}

	WSObj.moveStarted = false;
	WSObj.intervalHandler = false;
	WSObj.moveStart = function () {
		WSObj.moveStarted = true;

		var i, j;
		for (i in WSObj.objCallbackFunc) {
			var obj = WSObj.$(i);
			for (j in WSObj.moveMove[i]) {
				obj.style.top = (parseInt(obj.style.top) + WSObj.moveMove[i][j]) + 'px';
				delete WSObj.moveMove[i][j];
				if (j == WSObj.moveMove[i].length - 1) {
					WSObj.moveStop(i);
				}
				break;
			}
		}

		var objListIsEmpty = true;
		for (var i in WSObj.objCallbackFunc) {
			objListIsEmpty = false;
			break;
		}
		if (objListIsEmpty) {
			clearInterval(WSObj.intervalHandler);
			WSObj.intervalHandler = false;
			WSObj.moveStarted = false;
		} else if (!WSObj.intervalHandler) {
			// of course objListIsEmpty is false
			WSObj.intervalHandler = setInterval(WSObj.moveStart, WSObj.interval);
		}
	}

	WSObj.moveStop = function (objId) {
		WSObj.objCallbackFunc[objId]();

		delete WSObj.moveMove[objId];
		delete WSObj.objCallbackFunc[objId];
	}

	WSObj.illitionCount = 0;
	WSObj.illutionMove = function (objId, opacity, zIndex, callbackFunc, toY) {
		var getObj = WSObj.$(objId).cloneNode(true);
		var getObjOffset = WSObj.fetchOffset(objId);

		var illusionCount = WSObj.illitionCount;
		WSObj.illitionCount++;

		getObj.id = 'illusionMove_' + illusionCount;
		getObj.style.position = 'absolute';
		getObj.style.zIndex = zIndex;
		getObj.style.top = getObjOffset['top'] + 'px';
		getObj.style.left = getObjOffset['left'] + 'px';

		WSObj.setOpacity(objId, 0);

		WSObj.$(objId).parentNode.insertBefore(getObj, WSObj.$(objId));
		WSObj.setOpacity('illusionMove_' + illusionCount, opacity);

		WSObj.move('illusionMove_' + illusionCount, function() {
			WSObj.$(objId).parentNode.removeChild(WSObj.$('illusionMove_' + illusionCount));
			WSObj.setOpacity(objId, 1);
			callbackFunc();
		}, toY);

		return 'illusionMove_' + illusionCount;
	}

	WSObj.illutionMoveStop = function (objId) {
		WSObj.moveStop(objId);
	}

	WSObj.getStep = function (start, stop, speed, initSpeed) {
		initSpeed = typeof initSpeed != 'undefined' ? initSpeed : 1;

		var sign = start - stop;
		var distance = Math.abs(sign);
		var currentSpeed = initSpeed;

		var tempResult = [];
		var proceeded = 0, stepSize;
		while (proceeded < distance) {
			currentSpeed = currentSpeed * speed;
			stepSize = Math.ceil(Math.exp(currentSpeed));
			proceeded += stepSize;
			if (proceeded > distance) {
				stepSize -= (proceeded - distance);
				proceeded = distance;
			}
			tempResult[tempResult.length] = stepSize;
		}

		var returnResult = [];
		for (var i = tempResult.length - 1; i >= 0; i--) {
			returnResult[returnResult.length] = sign > 0 ? -tempResult[i] : tempResult[i];
		}

		return returnResult;
	}

    WSObj.fetchOffset = function(objId) {
        var obj = WSObj.$(objId);
		var offsetLeft = 0, offsetTop = 0;

		if(obj.getBoundingClientRect){
			var rect = obj.getBoundingClientRect();
			var scrollTop = Math.max(document.documentElement.scrollTop, document.body.scrollTop);
			var scrollLeft = Math.max(document.documentElement.scrollLeft, document.body.scrollLeft);
			if(document.documentElement.dir == 'rtl') {
				scrollLeft = scrollLeft + document.documentElement.clientWidth - document.documentElement.scrollWidth;
			}
			offsetLeft = rect.left + scrollLeft - document.documentElement.clientLeft;
			offsetTop = rect.top + scrollTop - document.documentElement.clientTop;
		}
		if(offsetLeft <= 0 || offsetTop <= 0) {
			offsetLeft = obj.offsetLeft;
			offsetTop = obj.offsetTop;
			while((obj = obj.offsetParent) != null) {
				offsetLeft += obj.offsetLeft;
				offsetTop += obj.offsetTop;
			}
		}

		return { 'left' : offsetLeft, 'top' : offsetTop };
    }

    WSObj.setOpacity = function(obj, opacityValue) {
        obj = WSObj.$(obj);
        if (WSObj.isIE && WSObj.isIE <= 8) {
            obj.style.filter = 'alpha(opacity=' + (opacityValue * 100) + ')';
        } else {
            obj.style.opacity = opacityValue;
        }
    }

    WSObj.$ = function (obj) {
        return typeof obj == 'string' ? document.getElementById(obj) : obj;
    }

	return WSObj;
}

// last revised: 13:26 9/8/2010
// to make it work fine, best pre-define the width of each block. To know more, run the demo on IE6 and Firefox and observe the difference.
// tested on fx3, fx3.5, fx3.6, ie6, ie7, ie8. ie9b4, chrome5, opera10, safari5, XHTML 1.0 Transitional

function ModuleFloatingDiv() {
	var WSObj = new Object();

    var userAgent = navigator.userAgent.toLowerCase();
    WSObj.isIE = window.ActiveXObject && userAgent.indexOf('msie') != -1 && userAgent.substr(userAgent.indexOf('msie') + 5, 3);

	// an object that store the data of those floating div
	WSObj.initData = new Object();
	// inputOptions = array('resize' => boolean, 'drag' => [''], 'heightExclude' => array('..', '...'))
	// drag and heightExclude are optional
	// best to include all elements except _content into heightExclude
	WSObj.show = function (objId, inputOptions) {
		var obj = WSObj.$(objId);
		objId = obj.id;

		if (obj.initialized) {
			WSObj.setOpacity(obj, 0);
			obj.style.display = '';
		} else {
			WSObj.init(obj, inputOptions);
		}

		var contentObj = WSObj.$(obj.id + '_content');
		var initWidth = WSObj.initData[objId]['initWidth'];

		obj.style.height = 'auto';
		contentObj.style.height = 'auto';

		// these three values need to be tested accross browsers since it varies between different browser and document type
		var documentClientHeight = document.documentElement.clientHeight;
		var documentClientWidth = document.documentElement.clientWidth;
		var documentScrollTop = document.documentElement.scrollTop == 0 ? document.documentElement.scrollTop + document.body.scrollTop : document.documentElement.scrollTop;

		var heightExceedClientHeight = obj.offsetHeight > documentClientHeight;
		if (heightExceedClientHeight) {
			obj.style.height = (documentClientHeight - 10) + 'px';
			contentObj.style.height = (parseInt(obj.offsetHeight) - WSObj.initData[obj.id]['inputOptions']['minusHeight'] - 22) + 'px';
		}

        // 20(px) for the scroll bar
		var widthExceedClientWidth = initWidth + 22 + (heightExceedClientHeight ? 20 : 0) > documentClientWidth;
		if (widthExceedClientWidth) {
			obj.style.width = (documentClientWidth - 20) + 'px';
		} else if (heightExceedClientHeight) {
			obj.style.width = (initWidth + 22 + 20) + 'px';
		} else {
		    obj.style.width = (initWidth + 22) + 'px';
        }
		contentObj.style.width = (parseInt(obj.offsetWidth) - 22) + 'px';

		obj.style.top = Math.floor(documentScrollTop + ((documentClientHeight - obj.offsetHeight) / 2)) + 'px';
		obj.style.left = Math.floor((documentClientWidth - obj.offsetWidth) / 2) + 'px';

		WSObj.zIndexToTop(objId);
		WSObj.setOpacity(obj, 1);
	}

	WSObj.init = function (obj, inputOptions) {
		if (obj.initialized == 1) {
			return true;
		}
		obj.initialized = 1;

		WSObj.initData[obj.id] = {};
		WSObj.initData[obj.id]['inputOptions'] = inputOptions;
		var width = WSObj.getWidth(obj, obj.innerHTML);
		WSObj.initData[obj.id]['initWidth'] = width;

		WSObj.setOpacity(obj, 0);
		obj.style.position = 'absolute';
		obj.style.display = '';
		obj.onmousedown =  function() { WSObj.zIndexToTop(obj.id); };

		var resize = typeof inputOptions['resize'] != 'undefined' && (inputOptions['resize'] == true || inputOptions['resize'] == 1) ? true : false;
		obj.innerHTML = '<table cellpadding="0" cellspacing="0" border="0" style="table-layout: fixed;">' +
								'<tr>' +
									'<td class="fd_tl" id="' + obj.id + '_fd_tl"' + (resize ? ' style="cursor: nw-resize;"' : '') + '></td>' +
									'<td class="fd_tbm" id="' + obj.id + '_fd_tm"' + (resize ? ' style="cursor: n-resize;"' : '') + '></td>' +
									'<td class="fd_tr" id="' + obj.id + '_fd_tr"' + (resize ? ' style="cursor: ne-resize;"' : '') + '></td>' +
								'</tr>' +
								'<tr>' +
									'<td class="fd_mlr" id="' + obj.id + '_fd_ml"' + (resize ? ' style="cursor: w-resize;"' : '') + '></td>' +
									'<td style="border: 1px solid #444; overflow: hidden;"><div id="' + obj.id + '_mainContainer">' + obj.innerHTML + '</div></td>' +
									'<td class="fd_mlr" id="' + obj.id + '_fd_mr"' + (resize ? ' style="cursor: w-resize;"' : '') + '></td>' +
								'</tr>' +
								'<tr>' +
									'<td class="fd_bl" id="' + obj.id + '_fd_bl"' + (resize ? ' style="cursor: ne-resize;"' : '') + '></td>' +
									'<td class="fd_tbm" id="' + obj.id + '_fd_bm"' + (resize ? ' style="cursor: n-resize;"' : '') + '></td>' +
									'<td class="fd_br" id="' + obj.id + '_fd_br"' + (resize ? ' style="cursor: nw-resize;"' : '') + '></td>' +
								'</tr>' +
							'</table>';
		if (resize) {
			WSObj.$(obj.id + '_fd_tl').onmousedown =  function(e) { WSObj.resizeEvent(obj.id, 'fd_tl', e, 1); };
			WSObj.$(obj.id + '_fd_tm').onmousedown =  function(e) { WSObj.resizeEvent(obj.id, 'fd_tm', e, 1); };
			WSObj.$(obj.id + '_fd_tr').onmousedown =  function(e) { WSObj.resizeEvent(obj.id, 'fd_tr', e, 1); };
			WSObj.$(obj.id + '_fd_ml').onmousedown =  function(e) { WSObj.resizeEvent(obj.id, 'fd_ml', e, 1); };
			WSObj.$(obj.id + '_fd_mr').onmousedown =  function(e) { WSObj.resizeEvent(obj.id, 'fd_mr', e, 1); };
			WSObj.$(obj.id + '_fd_bl').onmousedown =  function(e) { WSObj.resizeEvent(obj.id, 'fd_bl', e, 1); };
			WSObj.$(obj.id + '_fd_bm').onmousedown =  function(e) { WSObj.resizeEvent(obj.id, 'fd_bm', e, 1); };
			WSObj.$(obj.id + '_fd_br').onmousedown =  function(e) { WSObj.resizeEvent(obj.id, 'fd_br', e, 1); };
		}

		// initialize the properties of other element, to prevent unexpected lost of properties set (e.g. IE6).

		obj.style.overflow = 'hidden';
		WSObj.$(obj.id + '_content').style.overflow = 'auto';
		WSObj.initData[obj.id]['inputOptions']['minusHeight'] = 0;
		if (typeof inputOptions['heightExclude'] != 'undefined' && inputOptions['heightExclude'] != null) {
			for (var i in inputOptions['heightExclude']) {
				WSObj.$(obj.id + '_' + inputOptions['heightExclude'][i]).style.overflow = 'auto';
				// should set the obj.style.display='' first
				WSObj.initData[obj.id]['inputOptions']['minusHeight'] += WSObj.$(obj.id + '_' + inputOptions['heightExclude'][i]).offsetHeight;
			}
		}

		if (typeof inputOptions['drag'] != 'undefined') {
			for (var i in inputOptions['drag']) {
				WSObj.dragInit(obj.id, obj.id + '_' + inputOptions['drag'][i]);
			}
		}

		return true;
	}

	WSObj.close = function (objId) {
		for (var i in WSObj.zIndexData) {
			if (WSObj.zIndexData[i] == objId) {
				delete WSObj.zIndexData[i];
			}
		}
		WSObj.$(objId).style.display = 'none';
	}

	WSObj.dragInit = function (objId, dragObjId) {
		WSObj.$(dragObjId).style.cursor = 'move';
		WSObj.$(dragObjId).onmousedown = function (e) {
			WSObj.dragEvent(objId, e, 1);
		};
		WSObj.selectable(dragObjId, false);
	}

	WSObj.dragClientWidth = '';
	WSObj.dragClientHeight = '';
	WSObj.dragScrollTop = '';
	WSObj.dragInfo = new Array();
	WSObj.dragEvent = function (objId, e, operation) {
		var obj = WSObj.$(objId);

		if (operation == 1) {

			WSObj.dragClientWidth = document.documentElement.clientWidth;
			WSObj.dragClientHeight = document.documentElement.clientHeight;
			WSObj.dragScrollTop = document.documentElement.scrollTop == 0 ? document.documentElement.scrollTop + document.body.scrollTop : document.documentElement.scrollTop;

			// in IE9b4, if this is placed before the above three line, it will go to top:0px
			// however, I tried and still cannot reproduce the error in writing other script
			WSObj.selectable(document.documentElement, false);

			document.documentElement.onmousemove = function (e) {
				WSObj.dragEvent(objId, e, 2);
			}
			document.documentElement.onmouseup = function (e) {
				WSObj.dragEvent(objId, e, 3);
			}

			WSObj.dragInfo = WSObj.isIE ? [event.clientX, event.clientY] : [e.clientX, e.clientY];
			WSObj.dragInfo[2] = parseInt(obj.style.left);
			WSObj.dragInfo[3] = parseInt(obj.style.top);

		} else if (operation == 2) {

			var dragInfo = WSObj.isIE ? [event.clientX, event.clientY] : [e.clientX, e.clientY];

			var left = (WSObj.dragInfo[2] + dragInfo[0] - WSObj.dragInfo[0]);
			var top = (WSObj.dragInfo[3] + dragInfo[1] - WSObj.dragInfo[1]);
			left = left < 10 && left > -10 ? 0 : (left > WSObj.dragClientWidth - obj.offsetWidth - 10 && left < WSObj.dragClientWidth - obj.offsetWidth + 10 ? WSObj.dragClientWidth - obj.offsetWidth : left);
			top = top < WSObj.dragScrollTop + 10 && top > WSObj.dragScrollTop - 10 ? WSObj.dragScrollTop : (top > WSObj.dragScrollTop + WSObj.dragClientHeight - obj.offsetHeight - 10 && top < WSObj.dragScrollTop + WSObj.dragClientHeight - obj.offsetHeight + 10 ? WSObj.dragScrollTop + WSObj.dragClientHeight - obj.offsetHeight : top);

			obj.style.left = left + 'px';
			obj.style.top = top + 'px';

		} else if (operation == 3) {

			document.documentElement.onmousemove = function () { }
			document.documentElement.onmouseup = function () { }
			WSObj.selectable(document.documentElement, true);

		}
	}

	WSObj.resizeInfo = new Array();
	WSObj.resizeEvent = function (objId, type, e, operation) {
		var obj = WSObj.$(objId);
		if (operation == 1) {

			WSObj.selectable(document.documentElement, false);

			document.documentElement.onmousemove = function(e) { WSObj.resizeEvent(objId, type, e, 2); };
			document.documentElement.onmouseup = function(e) { WSObj.resizeEvent(objId, type, e, 3); };

			WSObj.resizeInfo = WSObj.isIE ? [event.clientX, event.clientY] : [e.clientX, e.clientY];
			WSObj.resizeInfo[2] = parseInt(obj.style.left);
			WSObj.resizeInfo[3] = parseInt(obj.style.top);
			WSObj.resizeInfo[4] = parseInt(obj.offsetWidth);
			WSObj.resizeInfo[5] = parseInt(obj.offsetHeight);

		} else if (operation == 2) {

			var resizeInfo = WSObj.isIE ? [event.clientX, event.clientY] : [e.clientX, e.clientY];
			var left = WSObj.resizeInfo[2];
			var top = WSObj.resizeInfo[3];
			var width = WSObj.resizeInfo[4];
			var height = WSObj.resizeInfo[5];

			if (type == 'fd_tl')
			{
				width = (WSObj.resizeInfo[4] - (resizeInfo[0] - WSObj.resizeInfo[0]));
				height = (WSObj.resizeInfo[5] - (resizeInfo[1] - WSObj.resizeInfo[1]));
				top = WSObj.resizeInfo[3] + WSObj.resizeInfo[5] - height;
				left = WSObj.resizeInfo[2] + WSObj.resizeInfo[4] - width;
			}
			else if (type == 'fd_tm')
			{
				width = WSObj.resizeInfo[4];
				height = (WSObj.resizeInfo[5] - (resizeInfo[1] - WSObj.resizeInfo[1]));
				top = WSObj.resizeInfo[3] + WSObj.resizeInfo[5] - height;
			}
			else if (type == 'fd_tr')
			{
				width = (WSObj.resizeInfo[4] + (resizeInfo[0] - WSObj.resizeInfo[0]));
				height = (WSObj.resizeInfo[5] - (resizeInfo[1] - WSObj.resizeInfo[1]));
				top = WSObj.resizeInfo[3] + WSObj.resizeInfo[5] - height;
			}
			else if (type == 'fd_ml')
			{
				width = (WSObj.resizeInfo[4] - (resizeInfo[0] - WSObj.resizeInfo[0]));
				left = WSObj.resizeInfo[2] + WSObj.resizeInfo[4] - width;
			}
			else if (type == 'fd_mr')
			{
				width = (WSObj.resizeInfo[4] + (resizeInfo[0] - WSObj.resizeInfo[0]));
			}
			else if (type == 'fd_bl')
			{
				width = (WSObj.resizeInfo[4] - (resizeInfo[0] - WSObj.resizeInfo[0]));
				height = (WSObj.resizeInfo[5] + (resizeInfo[1] - WSObj.resizeInfo[1]));
				left = WSObj.resizeInfo[2] + WSObj.resizeInfo[4] - width;
			}
			else if (type == 'fd_bm')
			{
				height = (WSObj.resizeInfo[5] + (resizeInfo[1] - WSObj.resizeInfo[1]));
			}
			else if (type == 'fd_br')
			{
				width = (WSObj.resizeInfo[4] + (resizeInfo[0] - WSObj.resizeInfo[0]));
				height = (WSObj.resizeInfo[5] + (resizeInfo[1] - WSObj.resizeInfo[1]));
			}

			width = width < 15 + 22 ? 15 + 22 : width;
			height = height < WSObj.initData[objId]['inputOptions']['minusHeight'] + 22 ? WSObj.initData[objId]['inputOptions']['minusHeight'] + 22 : height;

			if (top > WSObj.resizeInfo[3] + WSObj.resizeInfo[5] - height && (type == 'fd_tl' || type == 'fd_tm' || type == 'fd_tr')) {
				top = WSObj.resizeInfo[3] + WSObj.resizeInfo[5] - height;
			}
			if (left > WSObj.resizeInfo[2] + WSObj.resizeInfo[4] - width && (type == 'fd_tl' || type == 'fd_ml' || type == 'fd_bl')) {
				left = WSObj.resizeInfo[2] + WSObj.resizeInfo[4] - width;
			}

			obj.style.left = left + 'px';
			obj.style.top = top + 'px';
			obj.style.width = width + 'px';
			obj.style.height = height + 'px';

			WSObj.$(objId + '_content').style.height = (height - WSObj.initData[objId]['inputOptions']['minusHeight'] - 22) + 'px';
			WSObj.$(objId + '_content').style.width = (width - 22) + 'px';
			WSObj.$(objId + '_mainContainer').style.height = (height - 22) + 'px';
			WSObj.$(objId + '_mainContainer').style.width = (width - 22) + 'px';

		} else if (operation == 3) {

			document.documentElement.onmousemove = function() { };
			document.documentElement.onmouseup = function() { };
			WSObj.selectable(document.documentElement, true);

		}
	}

	WSObj.zIndexData = new Array();
	WSObj.zIndexToTop = function (objId) {
		for (var i in WSObj.zIndexData) {
			if (WSObj.zIndexData[i] == objId) {
				delete WSObj.zIndexData[i];
			}
		}
		WSObj.zIndexData[WSObj.zIndexData.length] = objId;
		var count = 10;
		for (var i in WSObj.zIndexData) {
			WSObj.$(WSObj.zIndexData[i]).style.zIndex = count;
			count++;
		}
	}

	WSObj.selectable = function (obj, state) {
		obj = WSObj.$(obj);
        // make the main object not selectable
        if (obj.style.MozUserSelect !== undefined) {
            obj.style.MozUserSelect = state ? 'all' : 'none';
        } else if (obj.style.webkitUserSelect !== undefined) {
            obj.style.webkitUserSelect = state ? 'auto' : 'none';
        } else {
            obj.unselectable = state ? 'off' : 'on';
            obj.onselectstart = state ? function () { } : function () { return false; };
        }
	}

    WSObj.getWidth = function(obj, innerHTML, className) {
        var getWidth = document.createElement('div');
        getWidth.style.position = 'absolute';
        WSObj.setOpacity(getWidth, 0);
        if (className) {
        	getWidth.className = className;
        } else {
        	getWidth.style.border = '0px';
        	getWidth.style.padding = '0px';
        }
        getWidth.innerHTML = innerHTML;
        obj.parentNode.insertBefore(getWidth, obj);
        var width = getWidth.offsetWidth;
        obj.parentNode.removeChild(getWidth);
        return width;
    }

    WSObj.setOpacity = function(obj, opacityValue) {
        obj = WSObj.$(obj);
        if (WSObj.isIE && WSObj.isIE <= 8) {
            obj.style.filter = 'alpha(opacity=' + (opacityValue * 100) + ')';
        } else {
            obj.style.opacity = opacityValue;
        }
    }

    WSObj.$ = function (obj) {
        return typeof obj == 'string' ? document.getElementById(obj) : obj;
    }

	return WSObj;
}

// last revised: 14:36 6/8/2010
// tested on ie9b4, chrome5, XHTML 1.0 Transitional

function ModuleForm() {
    var WSObj = new Object();

    WSObj.getElementValue = function (elementName, batch) {
    	batch = WSObj.isDefined(batch) ? batch : false;

        var i, j;

        var resultArray = {};
        var resultArrayCount = 0;

        if (batch) {
            var textareaTags = document.getElementsByTagName('textarea');
            for (i = 0; i < textareaTags.length; i++) {
                if (textareaTags[i].name.match(new RegExp("^" + elementName + "[[0-9a-zA-Z]+]$", "m"))) {
                    resultArray[textareaTags[i].name.substr(elementName.length + 1, textareaTags[i].name.length - elementName.length - 1 - 1)] = textareaTags[i].value;
                }
            }

            var elementTemp;

            var inputTags = document.getElementsByTagName('input');
            for (i = 0; i < inputTags.length; i++) {
                if (inputTags[i].type == 'text' || inputTags[i].type == 'hidden' || inputTags[i].type == 'password' || inputTags[i].type == 'radio') {
                    if (inputTags[i].name.match(new RegExp("^" + elementName + "[[0-9a-zA-Z]+]$", "m"))) {
                        if (inputTags[i].type == 'radio' && !inputTags[i].checked) {
                            continue;
                        }
                        resultArray[inputTags[i].name.substr(elementName.length + 1, inputTags[i].name.length - elementName.length - 1 - 1)] = inputTags[i].value;
                    }
                } else if (inputTags[i].type == 'checkbox') {
                    if (inputTags[i].name.match(new RegExp("^" + elementName + "[[0-9a-zA-Z]+]\\\[\\\]$", "m"))) {
                        elementTemp = inputTags[i].name.substr(elementName.length + 1, inputTags[i].name.length - elementName.length - 1 - 3);
                        if (inputTags[i].checked) {
	                        if (typeof resultArray[elementTemp] == 'undefined') {
	                            resultArray[elementTemp] = new Array();
	                        }
                            resultArray[elementTemp][resultArray[elementTemp].length] = inputTags[i].value;
                        }
                    }
                }
            }

            var selectTags = document.getElementsByTagName('select');
            for (i = 0; i < selectTags.length; i++) {
                if (selectTags[i].name.match(new RegExp("^" + elementName + "[[0-9a-zA-Z]+]$", "m"))) {
                    elementTemp = selectTags[i].name.substr(elementName.length + 1, selectTags[i].name.length - elementName.length - 1 - 1);
                    for (j = 0; j < selectTags[i].length; j++) {
                        if (selectTags[i][j].selected) {
		                    if (typeof resultArray[elementTemp] == 'undefined') {
		                        resultArray[elementTemp] = new Array();
		                    }
                            resultArray[elementTemp][resultArray[elementTemp].length] = selectTags[i][j].value;
                        }
                    }
                }
            }
        } else {
            var inputTags = document.getElementsByTagName('input');
            for (i = 0; i < inputTags.length; i++) {
                if (inputTags[i].type == 'checkbox') {
                    if ((inputTags[i].name == elementName + '[]' || inputTags[i].name == elementName) && inputTags[i].checked) {
                        resultArray[resultArrayCount] = inputTags[i].value;
                        resultArrayCount++;
                    }
                } else if (inputTags[i].type == 'text' || inputTags[i].type == 'hidden' || inputTags[i].type == 'password' || inputTags[i].type == 'radio') {
                    if (inputTags[i].name == elementName) {
                        if (inputTags[i].type == 'radio') {
                            if (inputTags[i].checked) {
                                return inputTags[i].value;
                            }
                        } else {
                            return inputTags[i].value;
                        }
                    }
                }
            }

            if (resultArrayCount == 0) {
                var textareaTags = document.getElementsByTagName('textarea');
                for (i = 0; i < textareaTags.length; i++) {
                    if (textareaTags[i].name == elementName) {
                        return textareaTags[i].value;
                    }
                }
            }

            if (resultArrayCount == 0) {
                var selectTags = document.getElementsByTagName('select');
                for (i = 0; i < selectTags.length; i++) {
                    if (selectTags[i].name == elementName) {
                        for (j = 0; j < selectTags[i].length; j++) {
                            if (selectTags[i][j].selected) {
                                resultArray[resultArrayCount] = selectTags[i][j].value;
                                resultArrayCount++;
                            }
                        }
                    }
                }
            }
        }

		var emptyArray = true;
		for (i in resultArray) {
			emptyArray = false;
			break;
		}

        return emptyArray ? false : resultArray;
    }

    WSObj.setElementValue = function (elementName, value) {
        var i, j, k, l;

        var fieldType = 'single';
        if (typeof value == 'object') {
            fieldType = 'multi';
            for (i in value) {
                if (typeof value[i] == 'object') {
                    fieldType = 'multiBatch';
                }
                break;
            }
        }

        var manipulated = false;

        var inputTags = document.getElementsByTagName('input');
        for (i = 0; i < inputTags.length; i++) {
            if (fieldType == 'single') {
                if (inputTags[i].name == elementName) {
                    if (inputTags[i].type == 'text' || inputTags[i].type == 'hidden' || inputTags[i].type == 'password') {
                        inputTags[i].value = value;
                        WSObj.triggerChangeFunction(inputTags[i]);
                        manipulated = true;
                    } else {
                        // alert('wrong input type');
                    }
                    break;
                }
            } else if (fieldType == 'multi') {
                if (inputTags[i].type == 'text' || inputTags[i].type == 'hidden' || inputTags[i].type == 'password') {
                    var elementLength = elementName.length;
                    if (inputTags[i].name.substr(0, elementLength) == elementName) {
                        for (j in value) {
                            if (inputTags[i].name == elementName + '[' + j + ']') {
                                inputTags[i].value = value[j];
                                WSObj.triggerChangeFunction(inputTags[i]);
                                manipulated = true;
                                break;
                            }
                        }
                    }
                } else if (inputTags[i].type == 'radio' || inputTags[i].type == 'checkbox') {
                    if (inputTags[i].name == elementName || (inputTags[i].type == 'checkbox' && inputTags[i].name == elementName + '[]')) {
                        for (j in value) {
                            inputTags[i].checked = false;
                            if (inputTags[i].value == value[j]) {
                                inputTags[i].checked = true;
                                WSObj.triggerChangeFunction(inputTags[i]);
                                manipulated = true;
                                break;
                            }
                        }
                    }
                }
            } else if (fieldType == 'multiBatch') {
                //if (inputTags[i].type != 'radio' && inputTags[i].type != 'checkbox') {
                //    alert('wrong input type');
                //}
                var elementLength = elementName.length;
                if (inputTags[i].name.substr(0, elementLength) == elementName) {
                    for (j in value) {
                        for (k in value[j]) {
                            if (inputTags[i].name == elementName + '[' + j + ']' || (inputTags[i].type == 'checkbox' && inputTags[i].name == elementName + '[' + j + '][]')) {
                                if (inputTags[i].value == value[j][k]) {
                                    inputTags[i].checked = true;
                                    WSObj.triggerChangeFunction(inputTags[i]);
                                    manipulated = true;
                                    break;
                                }
                            }
                        }
                    }
                }
            }
        }

        if (!manipulated) {
            var selectTags = document.getElementsByTagName('select');
            for (i = 0; i < selectTags.length; i++) {
                if (fieldType == 'multi') {
                    //if (selectTags[i].type != 'select-one') {
                    //    alert('wrong input type');
                    //}
                    if (selectTags[i].name == elementName) {
                        for (j = 0; j < selectTags[i].length; j++) {
                            for (k in value) {
                                selectTags[i][j].selected = false;
                                if (value[k] == selectTags[i][j].value) {
                                    selectTags[i][j].selected = true;
                                    WSObj.triggerChangeFunction(selectTags[i][j]);
                                    break;
                                }
                            }
                        }
                    }
                } else if (fieldType == 'multiBatch') {
                    //if (selectTags[i].type != 'select-multiple') {
                    //    alert('wrong input type');
                    //}
                    var elementLength = elementName.length;
                    if (selectTags[i].name.substr(0, elementLength) == elementName) {
                        for (k in value) {
                            if (selectTags[i].name == elementName + '[' + k + ']') {
                                for (j = 0; j < selectTags[i].length; j++) {
                                    for (l in value[k]) {
                                        selectTags[i][j].selected = false;
                                        if (value[k][l] == selectTags[i][j].value) {
                                            selectTags[i][j].selected = true;
                                            WSObj.triggerChangeFunction(selectTags[i][j]);
                                            manipulated = true;
                                            break;
                                        }
                                    }
                                }
                            }
                        }
                    }
                }
            }
        }

        if (!manipulated) {
            var textareaTags = document.getElementsByTagName('textarea');
            for (i = 0; i < textareaTags.length; i++) {
                if (fieldType == 'single') {
                    if (textareaTags[i].name == elementName) {
                        textareaTags[i].value = value;
                        WSObj.triggerChangeFunction(textareaTags[i]);
                        manipulated = true;
                        break;
                    }
                } else {
                    var elementLength = elementName.length;
                    if (textareaTags[i].name.substr(0, elementLength) == elementName) {
                        for (j in value) {
                            if (textareaTags[i].name == elementName + '[' + j + ']') {
                                textareaTags[i].value = value[j];
                                WSObj.triggerChangeFunction(textareaTags[i]);
                                manipulated = true;
                                break;
                            }
                        }
                    }
                }
            }
        }
    }

    WSObj.triggerChangeFunction = function (obj) {
        if (WSObj.isDefined(obj.onclick)) obj.onclick();
        if (WSObj.isDefined(obj.onchange)) obj.onchange();
        if (WSObj.isDefined(obj.onblur)) obj.onblur();
    }

	WSObj.formToString = function (formName, prefix) {
		prefix = WSObj.isDefined(prefix) ? prefix : '';

		var params = '';

		for (var i in document.forms) {
			if (document.forms[i].name != formName) continue;

			var name, k;
			for (var j = 0; j < document.forms[i].elements.length; j++) {
				if (document.forms[i].elements[j].name) {
					name = document.forms[i].elements[j].name.substr(prefix.length, document.forms[i].elements[j].name.length - prefix.length);

					if (document.forms[i].elements[j].tagName.toLowerCase() == 'input') {
						if (document.forms[i].elements[j].type == 'radio' || document.forms[i].elements[j].type == 'checkbox') {
							if (document.forms[i].elements[j].checked) {
								params += name + '=' + encodeURIComponent(document.forms[i].elements[j].value) + '&';
							}
						} else {
							params += name + '=' + encodeURIComponent(document.forms[i].elements[j].value) + '&';
						}
					} else if (document.forms[i].elements[j].tagName.toLowerCase() == 'textarea') {
						params += name + '=' + encodeURIComponent(document.forms[i].elements[j].value) + '&';
					} else if (document.forms[i].elements[j].tagName.toLowerCase() == 'select') {
                        for (k = 0; k < document.forms[i].elements[j].length; k++) {
                            if (document.forms[i].elements[j][k].selected) {
                            	if (document.forms[i].elements[j].type == 'select-one') {
                            		params += name + '=' + encodeURIComponent(document.forms[i].elements[j].value) + '&';
                              	} else {
                            		params += name + '[]=' + encodeURIComponent(document.forms[i].elements[j].value) + '&';
                              	}
                            }
                        }
					}
				}
			}

			break;
		}

		return params;
	}

	WSObj.formToArray = function (formName, prefix) {
		prefix = WSObj.isDefined(prefix) ? prefix : '';

		var returnArray = {};
		var fieldIncluded = [];

		for (var i in document.forms) {
			if (document.forms[i].name != formName) continue;

			var name, value, k;
			for (var j = 0; j < document.forms[i].elements.length; j++) {
				if (document.forms[i].elements[j].name) {
					k = document.forms[i].elements[j].name.indexOf('[');
					name = k != -1 ? document.forms[i].elements[j].name.substr(0, k) : document.forms[i].elements[j].name;

					if (WSObj.inArray(name, fieldIncluded)) {
						continue;
					}
					fieldIncluded[fieldIncluded.length] = name;

					value = false;
					if (document.forms[i].elements[j].tagName.toLowerCase() == 'input' && document.forms[i].elements[j].type == 'checkbox') {
						if (document.forms[i].elements[j].name.match(new RegExp("^" + name + "[[0-9a-zA-Z]+]\\\[\\\]$", "m"))) {
							value = WSObj.getElementValue(name, true);
						} else {
							value = WSObj.getElementValue(name, false);
						}
					} else {
						if (document.forms[i].elements[j].name.match(new RegExp("^" + name + "[[0-9a-zA-Z]+]$", "m"))) {
							value = WSObj.getElementValue(name, true);
						} else {
							value = WSObj.getElementValue(document.forms[i].elements[j].name, false);
						}
					}

					if (value !== false) {
						returnArray[name] = value;
					}
				}
			}

			break;
		}

		return returnArray;
	}

	WSObj.inArray = function(key, array) {
		for (var i in array) {
			if (key == array[i]) {
				return true;
			}
		}
		return false;
	}

    WSObj.isDefined = function (target) {
        return typeof target != 'undefined' && target != null;
    }

    return WSObj;
}

// last revised: 13:28 9/8/2010
// it will only trigger the onchange event of the original select element, event of onclick, onmousedown etc. will not be triggered
// tested on fx3, fx3.5, fx3.6, ie6, ie7, ie8, ie9b4, chrome5, opera10, safari5, XHTML 1.0 Transitional

function ModuleFormDecoration() {
    var WSObj = new Object();

    var userAgent = navigator.userAgent.toLowerCase();
    WSObj.isIE = window.ActiveXObject && userAgent.indexOf('msie') != -1 && userAgent.substr(userAgent.indexOf('msie') + 5, 3);

    /**
     * Determine whether the document.documentElement.onmousedown event has been initialized or not
     */
    WSObj.documentOnmousedownEvent = false;

    /**
     * Use the property of global variable.
     * When one is active and at that time you click on the other select element, we use this to store the object id of current clicking element.
     * It is used by selectHide().
     * It is used accompany with selectActive.
     */
    WSObj.currentClick = '';
    /**
     * Store the object id of the current active select element
     */
    WSObj.selectActive = '';

    /**
     * Store the setting data of the select element. e.g. array(objId1 => array('inputOptions' => array()));
     */
    WSObj.selectData = new Object();

    WSObj.selectInit = function (objId, inputOptions) {
        if (typeof objId == 'object' || typeof objId == 'array') {
            for (var i in objId) {
                WSObj.selectInitSingle(objId[i], inputOptions);
            }
        } else {
            WSObj.selectInitSingle(objId, inputOptions);
        }
    }

    WSObj.selectInitSingle = function (objId, inputOptions) {
        var obj = WSObj.$(objId);
        objId = obj.id;

        // if the class of onmouseover is not set, set the class to normal class
        if (!inputOptions['main_mouseover']) {
            inputOptions['main_mouseover'] = inputOptions['main_normal'];
        }

        // set the onchange event on the object, to make the value on the select button and this decoration element synchronize
        var onchangeReserve = obj.onchange != null && typeof obj.onchange != 'undefined' ? obj.onchange : function () {};
        obj.onchange = function () {
            onchangeReserve();
            WSObj.selectOnchange(objId, this.selectedIndex);
        }

        // add the document.documentElement.onmousedown event
        if (!WSObj.documentOnmousedownEvent) {
            WSObj.documentOnmousedownEvent = true;
            var onmousedownPreserve = document.documentElement.onmousedown != null && typeof document.documentElement.onmousedown != 'undefined' ? document.documentElement.onmousedown : function () {};
            document.documentElement.onmousedown = function () {
                WSObj.selectHide();
                onmousedownPreserve();
            }
        }

        // store the setting data
        WSObj.selectData[objId] = new Object();
        WSObj.selectData[objId]['inputOptions'] = inputOptions;

        /**
         * Create the div for the select
         */
        var arrowImg = '<img src="'+inputOptions['icon_normal'][1]+'" class="'+inputOptions['icon_normal'][0]+'" style="width: '+inputOptions['icon_normal'][2]+'px; float: right;" id="fDeco_selectImg_'+objId+'" />';

        // the main element
        var elementMain = document.createElement('div');
        elementMain.id = 'fDeco_sMain_' + objId;
        elementMain.innerHTML = arrowImg + obj[obj.selectedIndex].text;
        WSObj.selectInitMainEvent(objId, elementMain);

        // the list element
        var elementList = document.createElement('div');
        elementList.id = 'fDeco_sList_' + objId;
        elementList.className = inputOptions['list_box'];
        elementList.style.position = 'absolute';
        elementList.onmousedown = function (event) { WSObj.doane(event); }


        // get the maximum width of the arrow image
        // for compatibility of Fx3, use div tag instead of img tag
        var imgWidth = WSObj.getWidth(obj, '<div class="'+inputOptions['icon_normal'][0]+'" style="width:'+inputOptions['icon_normal'][2]+'px"></div>');

        // store the image width
        WSObj.selectData[objId]['imgWidth'] = imgWidth;


        // get the maximum width of the element
        var innerHTML = '';
        for (var i = 0; i < obj.length; i++) {
            innerHTML += '<br />' + obj[i].text;
        }
        var width = WSObj.getWidth(obj, innerHTML, inputOptions['main_normal']) + imgWidth;
        // width + 1 for IE9b4
        if (WSObj.isIE && WSObj.isIE >= 9) {
        	width += 1;
        }


        // create the element list
        WSObj.selectData[objId]['elementContent'] = new Array();
        var itemElement;
        for (var i = 0; i < obj.length; i++) {
            itemElement = document.createElement('div');
            itemElement.id = 'fDeco_sList_' + objId + '_item' + i;
            itemElement.className = inputOptions['list_item_normal'];
            itemElement.innerHTML = obj[i].text;
            // set width: compatibility changes for IE6
            if (WSObj.isIE <= 6) {
            	itemElement.style.width = '100%';
            } else {
            	itemElement.style.width = 'auto';
            }
            itemElement.onmouseover = function () {
                this.className = this.className.replace(inputOptions['list_item_normal'], inputOptions['list_item_mouseover']);
            }
            itemElement.onmouseout = function () {
                this.className = this.className.replace(inputOptions['list_item_mouseover'], inputOptions['list_item_normal']);
            }
            WSObj.selectSetOnchange(objId, itemElement, i);
            elementList.appendChild(itemElement);

            WSObj.selectData[objId]['elementContent'][i] = obj[i].text;
        }


        elementMain.style.width = width + 'px';
        elementList.style.width = width + 'px';

        obj.parentNode.insertBefore(elementList, obj);
        obj.parentNode.insertBefore(elementMain, obj);

        if (elementMain.offsetWidth != elementList.offsetWidth) {
            elementList.style.width = (width + elementMain.offsetWidth - elementList.offsetWidth) + 'px';
        }

        // do the 'fit' action
        var elementMainUseMaxWidth = typeof inputOptions['fit'] == 'undefined' || !inputOptions['fit'] ? false : true;
        if (elementMainUseMaxWidth) {
            if (!WSObj.isIE || WSObj.isIE > 7) {
                elementMain.style.width = 'auto';
            } else {
                var textWidth = WSObj.getWidth(obj, obj[obj.selectedIndex].text, inputOptions['main_normal']);
                elementMain.style.width = (textWidth + imgWidth) + 'px';
            }
        }

        if (WSObj.isIE && WSObj.isIE <= 7) {
            WSObj.$('fDeco_sMain_' + objId).style.display = 'inline';
            WSObj.$('fDeco_sMain_' + objId).style.zoom = '1';
        } else {
            WSObj.$('fDeco_sMain_' + objId).style.display = 'inline-block';
        }

        obj.style.display = 'none';
        elementList.style.display = 'none';
    }

    WSObj.selectSetOnchange = function (mainObjId, eventObj, pos) {
        eventObj.onclick = function () {
            WSObj.selectOnchange(mainObjId, pos);
        }
    }

    WSObj.selectOnchange = function (objId, pos) {
        if (pos != WSObj.$(objId).selectedIndex) {
            WSObj.$(objId).selectedIndex = pos;
            if (typeof WSObj.$(objId).onchange != 'undefined' && WSObj.$(objId).onchange != null) {
                WSObj.$(objId).onchange();
            }
        }

        // compatibility changes for IE6, IE7
        if (WSObj.isIE && WSObj.isIE <= 7 && WSObj.selectData[objId]['inputOptions']['fit']) {
            var textWidth = WSObj.getWidth(WSObj.$('fDeco_sMain_' + objId), WSObj.selectData[objId]['elementContent'][pos], WSObj.selectData[objId]['inputOptions']['main_normal']);
            WSObj.$('fDeco_sMain_' + objId).style.width = (textWidth + WSObj.selectData[objId]['imgWidth']) + 'px';
        }
        WSObj.$('fDeco_sMain_' + objId).innerHTML = '<img src="'+WSObj.selectData[objId]['inputOptions']['icon_normal'][1]+'" class="'+WSObj.selectData[objId]['inputOptions']['icon_normal'][0]+'" style="width: '+WSObj.selectData[objId]['inputOptions']['icon_normal'][2]+'px; float: right;" id="fDeco_selectImg_'+objId+'" />' + WSObj.selectData[objId]['elementContent'][pos];

        WSObj.selectHide();
    }

    WSObj.selectInitMainEvent = function (objId, elementMain) {
        var inputOptions = WSObj.selectData[objId]['inputOptions'];

        // initialize the class of the main object
        elementMain.className = inputOptions['main_normal'];

        // make the main object not selectable
        if (elementMain.style.MozUserSelect !== undefined) {
            elementMain.style.MozUserSelect = 'none';
        } else if (elementMain.style.webkitUserSelect !== undefined) {
            elementMain.style.webkitUserSelect = 'none';
        } else {
        	elementMain.unselectable = 'on';
            elementMain.onselectstart = function () { return false; }
        }

        // as it is newly created, there should have no event exist
        elementMain.onmouseover = function () {
            elementMain.className = elementMain.className.replace(inputOptions['main_normal'], inputOptions['main_mouseover']);
            WSObj.selectChangeIcon(objId, 'active');
        }
        elementMain.onmouseout = function () {
            elementMain.className = elementMain.className.replace(inputOptions['main_mouseover'], inputOptions['main_normal']);
            WSObj.selectChangeIcon(objId, 'normal');
        }
        elementMain.onclick = function () {
            elementMain.className = elementMain.className.replace(inputOptions['main_mouseover'], inputOptions['main_click']).replace(inputOptions['main_normal'], inputOptions['main_click']);
            WSObj.selectChangeIcon(objId, 'active');
            WSObj.selectShow(objId);
        }
    }

    /**
     * Change the state of the img icon according to the objId.
     */
    WSObj.selectChangeIcon = function (objId, state) {
        if (state == 'active') {
            WSObj.$('fDeco_selectImg_' + objId).src = WSObj.selectData[objId]['inputOptions']['icon_active'][1];
            WSObj.$('fDeco_selectImg_' + objId).className = WSObj.selectData[objId]['inputOptions']['icon_active'][0];
        } else if (state == 'normal') {
            WSObj.$('fDeco_selectImg_' + objId).src = WSObj.selectData[objId]['inputOptions']['icon_normal'][1];
            WSObj.$('fDeco_selectImg_' + objId).className = WSObj.selectData[objId]['inputOptions']['icon_normal'][0];
        }
    }

    WSObj.selectShow = function (objId) {
        if (WSObj.selectActive != objId) {
            WSObj.selectHide();
        }

        // to make the zIndex take effect
        var mainObjClone = WSObj.$('fDeco_sMain_' + objId).cloneNode(true);
        mainObjClone.id = 'fDeco_sMain_clone_' + objId;
        mainObjClone.style.position = 'absolute';
        var offset = WSObj.fetchOffset('fDeco_sMain_' + objId);
        mainObjClone.style.top = offset['top'] + 'px';
        mainObjClone.style.left = offset['left'] + 'px';
        mainObjClone.style.margin = '0px';
        mainObjClone.style.zIndex = 10;
        mainObjClone.onmousedown = function () {
            WSObj.currentClick = objId;
        }
        WSObj.$('fDeco_sMain_' + objId).parentNode.insertBefore(mainObjClone, WSObj.$('fDeco_sMain_' + objId));

        WSObj.$('fDeco_sList_' + objId).style.top = (offset['top'] + WSObj.$('fDeco_sMain_' + objId).offsetHeight - 1) + 'px';
        WSObj.$('fDeco_sList_' + objId).style.left = offset['left'] + 'px';
        WSObj.$('fDeco_sList_' + objId).style.zIndex = 9;
        WSObj.$('fDeco_sList_' + objId).style.display = '';
        for (var i in WSObj.selectData[objId]['elementContent']) {
            WSObj.$('fDeco_sList_' + objId + '_item' + i).style.display = i == WSObj.$(objId).selectedIndex ? 'none' : '';
        }

		WSObj.setOpacity('fDeco_sMain_' + objId, 0);
        WSObj.selectActive = objId;
    }

    WSObj.selectHide = function () {
        if (WSObj.selectActive != '' && (WSObj.currentClick == '' || WSObj.currentClick != WSObj.selectActive)) {
			WSObj.setOpacity('fDeco_sMain_' + WSObj.selectActive, 1);
            WSObj.$('fDeco_sList_' + WSObj.selectActive).parentNode.removeChild(WSObj.$('fDeco_sMain_clone_' + WSObj.selectActive));
            WSObj.$('fDeco_sMain_' + WSObj.selectActive).className = WSObj.selectData[WSObj.selectActive]['inputOptions']['main_normal'];
            WSObj.$('fDeco_sList_' + WSObj.selectActive).style.display = 'none';
            WSObj.selectChangeIcon(WSObj.selectActive, 'normal');
            WSObj.selectActive = '';
        }
        if (WSObj.currentClick == WSObj.selectActive) {
            WSObj.currentClick = '';
        }
    }

    WSObj.getWidth = function(obj, innerHTML, className) {
        var getWidth = document.createElement('div');
        getWidth.style.position = 'absolute';
        if (className) {
        	getWidth.className = className;
        }
        getWidth.style.border = '0px';
        getWidth.style.padding = '0px';
        WSObj.setOpacity(getWidth, 0);
        getWidth.innerHTML = innerHTML;
        obj.parentNode.insertBefore(getWidth, obj);
        var width = getWidth.offsetWidth;
        obj.parentNode.removeChild(getWidth);
        return width;
    }

    WSObj.doane = function (event) {
        e = event ? event : window.event;
        if (WSObj.isIE) {
            e.cancelBubble = true;
        } else {
            e.stopPropagation();
        }
    }

    WSObj.fetchOffset = function(objId) {
        var obj = WSObj.$(objId);
		var offsetLeft = 0, offsetTop = 0;

		if(obj.getBoundingClientRect){
			var rect = obj.getBoundingClientRect();
			var scrollTop = Math.max(document.documentElement.scrollTop, document.body.scrollTop);
			var scrollLeft = Math.max(document.documentElement.scrollLeft, document.body.scrollLeft);
			if(document.documentElement.dir == 'rtl') {
				scrollLeft = scrollLeft + document.documentElement.clientWidth - document.documentElement.scrollWidth;
			}
			offsetLeft = rect.left + scrollLeft - document.documentElement.clientLeft;
			offsetTop = rect.top + scrollTop - document.documentElement.clientTop;
		}
		if(offsetLeft <= 0 || offsetTop <= 0) {
			offsetLeft = obj.offsetLeft;
			offsetTop = obj.offsetTop;
			while((obj = obj.offsetParent) != null) {
				offsetLeft += obj.offsetLeft;
				offsetTop += obj.offsetTop;
			}
		}

		return { 'left' : offsetLeft, 'top' : offsetTop };
    }

    WSObj.setOpacity = function(obj, opacityValue) {
        obj = WSObj.$(obj);
        if (WSObj.isIE && WSObj.isIE <= 8) {
            obj.style.filter = 'alpha(opacity=' + (opacityValue * 100) + ')';
        } else {
            obj.style.opacity = opacityValue;
        }
    }

    WSObj.$ = function (obj) {
        return typeof obj == 'string' ? document.getElementById(obj) : obj;
    }

    return WSObj;
}

// last revised: 14:12 5/8/2010
// tested on fx3, fx3.5, fx3.6, ie6, ie7, ie8, ie9b4, chrome5, opera10, safari5, XHTML 1.0 Transitional

function ModuleHiddenInput() {
	var WSObj = new Object();

    var userAgent = navigator.userAgent.toLowerCase();
    WSObj.isIE = window.ActiveXObject && userAgent.indexOf('msie') != -1 && userAgent.substr(userAgent.indexOf('msie') + 5, 3);

	WSObj.documentElementEventInitialized = false;
	/**
	 * Public.
	 */
	WSObj.init = function (objId, hiddenClassName, shownClassName, onMouseOverClassName) {
		if (!WSObj.documentElementEventInitialized) {
			WSObj.documentElementEventInitialized = true;
			var preserveEvent = document.documentElement.onmousedown != null ? document.documentElement.onmousedown : function () {};
			document.documentElement.onmousedown = function (e) {
				preserveEvent(e);
				WSObj.hideAll();
			}
		}

		if (typeof objId == 'array' || typeof objId == 'object') {
			for (var i in objId) {
				WSObj.initSingle(objId[i], hiddenClassName, shownClassName, onMouseOverClassName);
			}
		} else {
			WSObj.initSingle(objId, hiddenClassName, shownClassName, onMouseOverClassName);
		}
	}

	WSObj.data = {};
	WSObj.current = '';
	/**
	 * Protected
	 */
	WSObj.initSingle = function (objId, hiddenClassName, shownClassName, onMouseOverClassName) {
		var obj = WSObj.$(objId);
		objId = obj.id;

		if (typeof onMouseOverClassName == 'undefined') {
			onMouseOverClassName = shownClassName;
		}

		WSObj.data[objId] = [objId, hiddenClassName, shownClassName, onMouseOverClassName];

		var preserveEvent = obj.onmouseover != null ? obj.onmouseover : function () {};
		obj.onmouseover = function (e) {
			preserveEvent(e);
			if (WSObj.current != this.id) {
				this.className = this.className.replace(hiddenClassName, onMouseOverClassName);
			}
			// new title as tooltip
			this.title=this.value;
		}
		preserveEvent = obj.onmouseout != null ? obj.onmouseout : function () {};
		obj.onmouseout = function (e) {
			preserveEvent(e);
			if (WSObj.current != this.id) {
				this.className = this.className.replace(onMouseOverClassName, hiddenClassName);
			}
		}
		preserveEvent = obj.onmousedown != null ? obj.onmousedown : function () {};
		obj.onmousedown = function (e) {
			preserveEvent(e);
			if (WSObj.current == this.id) {
				WSObj.doane(e);
			}
		}
		preserveEvent = obj.onclick != null ? obj.onclick : function () {};
		obj.onclick = function (e) {
			preserveEvent(e);
			if (WSObj.current != this.id) {
				this.select();
			}
			WSObj.current = this.id;
			this.className = this.className.replace(hiddenClassName, shownClassName).replace(onMouseOverClassName, shownClassName);
		}
		
		// new 
		preserveEvent = obj.onkeypress != null ? obj.onkeypress : function () {};
		obj.onkeypress = function (e) {
			// preserveEvent(e);
			// WSObj.current = this.id;
			
			var key;
			var keychar;

			if (window.event)
				key = window.event.keyCode;
			else if (e)
				key = e.which;
			else
				return true;
			keychar = String.fromCharCode(key);
			
			// control keys
			if(key==27){ // Esc
				this.className = this.className.replace(onMouseOverClassName, hiddenClassName);
				this.blur();
			}
		}
		
	}

	/**
	 * Public but not for user to use.
	 */
	WSObj.hideAll = function () {
		if (WSObj.current != '') {
			var obj = WSObj.$(WSObj.current);
			obj.className = obj.className.replace(WSObj.data[WSObj.current][2], WSObj.data[WSObj.current][1]);
			WSObj.current = '';
		}
	}

	/**
	 * Protected.
	 */
	WSObj.doane = function (event) {
		e = event ? event : window.event;
		if (WSObj.isIE) {
			e.cancelBubble = true;
		} else {
			e.stopPropagation();
		}
	}
	
	/**
	 * Public.
	 */
	WSObj.setCurrent = function (objId) {
		var obj = WSObj.$(objId);
		objId = obj.id;
		WSObj.current = objId;
	}
	
	WSObj.setCurrent2 = function (obj) {
		objId = obj.id;
		WSObj.current = objId;
	}
	
	WSObj.getCurrent = function () {
		return WSObj.current;
	}

	/**
	 * Protected.
	 */
    WSObj.$ = function (obj) {
        return typeof obj == 'string' ? document.getElementById(obj) : obj;
    }

	return WSObj;
}

// last revised: 13:46 17/8/2010
// tested on fx2, fx3, fx3.5, fx3.6, fx4b3, ie6, ie7, ie8, ie9b4, chrome5, opera10, safari5, XHTML 1.0 Transitional

function ModuleSmartPanel() {
	var WSObj = {};

	var userAgent = navigator.userAgent.toLowerCase();
	WSObj.isIE = window.ActiveXObject && userAgent.indexOf('msie') != -1 && userAgent.substr(userAgent.indexOf('msie') + 5, 3);

	WSObj.mainEventInitialized = false;

	WSObj.counter = 0;
	
	// added by chris
	WSObj.zIndex = 8;
	
	WSObj.setParm = function (zIndex) {
		WSObj.zIndex = zIndex;
	}

	/**
	 * getElementFunc:
	 * return HTML or id/element?
	 *  - IE6 div width=100% for position:absolute by default -> don't create a new element to contain it
	 *  * Result: return id/element
	 */

	// mode: top|bottom|both
	WSObj.init = function (mode, getElementFunc, observeTop, observeBottom) {
		if (!WSObj.mainEventInitialized) {
			WSObj.mainEventInitialized = true;
			var preserveEvent = typeof window.onresize != 'undefined' && window.onresize != null ? window.onresize : function () {};
			window.onresize = function () {
				preserveEvent();
				WSObj.onchangeEvent('resize');
			}
			preserveEvent = typeof window.onscroll != 'undefined' && window.onscroll != null ? window.onscroll : function () {};
			window.onscroll = function () {
				preserveEvent();
				WSObj.onchangeEvent();
			}
		}

		WSObj.counter++;
		WSObj.initData(WSObj.counter, mode, getElementFunc, observeTop, observeBottom);
	}

	WSObj.initData = function (counter, mode, getElementFunc, observeTop, observeBottom) {
		WSObj.objData[counter] = {};

		WSObj.objData[counter]['state'] = 'hide';
		WSObj.objData[counter]['mode'] = mode;
		WSObj.objData[counter]['element'] = WSObj.$(getElementFunc(mode));

		WSObj.objData[counter]['observeTopOffset'] = 0;
		if (typeof observeTop != 'undefined') {
			if (typeof observeTop != 'string') {
				WSObj.objData[counter]['observeTop'] = observeTop[0];
				WSObj.objData[counter]['observeTopOffset'] = observeTop[1];
			} else {
				WSObj.objData[counter]['observeTop'] = observeTop;
			}
		} else {
			WSObj.objData[counter]['observeTop'] = '';
		}

		WSObj.objData[counter]['observeBottomOffset'] = 0;
		if (typeof observeBottom != 'undefined') {
			if (typeof observeBottom != 'string') {
				WSObj.objData[counter]['observeBottom'] = observeBottom[0];
				WSObj.objData[counter]['observeBottomOffset'] = observeBottom[1];
			} else {
				WSObj.objData[counter]['observeBottom'] = observeBottom;
			}
		} else {
			WSObj.objData[counter]['observeBottom'] = '';
		}

		WSObj.objData[counter]['element'].style.position = !WSObj.isIE || WSObj.isIE >= 7 ? 'fixed' : 'absolute';
		WSObj.objData[counter]['element'].style.zIndex = WSObj.zIndex;
		WSObj.objData[counter]['element'].style.display = '';
		WSObj.objData[counter]['elementOffsetHeight'] = WSObj.objData[counter]['element'].offsetHeight;
		WSObj.objData[counter]['element'].style.display = 'none';
		var scrollLeft = Math.max(document.documentElement.scrollLeft, document.body.scrollLeft);
		var clientLeft = document.documentElement.clientLeft;
		WSObj.objData[counter]['element'].style.left = (WSObj.objData[counter]['mode'] == 'top' ? WSObj.fetchOffsetLeft(WSObj.objData[counter]['observeTop'], scrollLeft, clientLeft) : WSObj.fetchOffsetLeft(WSObj.objData[counter]['observeBottom'], scrollLeft, clientLeft)) + 'px';
	}

	// array(1 => array('state' => 'hide'|shown', 'mode' => mode, 'element' => obj, 'observeTop' => observeTop, 'observeBottom' => observeBottom), 2 => array('state' => 'hide'|shown', 'mode' => mode, 'element' => obj, 'observeTop' => observeTop, 'observeBottom' => observeBottom))
	WSObj.objData = {};

	WSObj.eventTriggerTime = 0;

	WSObj.onchangeEvent = function (type) {
		if (!WSObj.isIE || WSObj.isIE != 6) {
			var time = (new Date()).getTime();
			if (time - WSObj.eventTriggerTime < 50) {
				return;
			}
			WSObj.eventTriggerTime = time;
		}
		type = typeof type == 'undefined' ? '' : type;

		var scrollTop = Math.max(document.documentElement.scrollTop, document.body.scrollTop);
		var clientTop = document.documentElement.clientTop;
		var clientHeight = !WSObj.isIE || WSObj.isIE >= 7 ? 0 : document.documentElement.clientHeight;

		for (var i in WSObj.objData) {
			var elementData = WSObj.objData[i];

			if (type == 'resize') {
				var scrollLeft = Math.max(document.documentElement.scrollLeft, document.body.scrollLeft);
				var clientLeft = document.documentElement.clientLeft;
				elementData['element'].style.left = (elementData['mode'] == 'top' ? WSObj.fetchOffsetLeft(elementData['observeTop'], scrollLeft, clientLeft) : WSObj.fetchOffsetLeft(elementData['observeBottom'], scrollLeft, clientLeft)) + 'px';
			}

			switch (elementData['mode']) {
				case 'top':
					if (elementData['observeBottom'] != '') {
						var observeBottom = WSObj.fetchOffsetTop(elementData['observeBottom'], scrollTop, clientTop);
						if (scrollTop + elementData['elementOffsetHeight'] + elementData['observeTopOffset'] > observeBottom + WSObj.$(elementData['observeBottom']).offsetHeight) {
							WSObj.hide(i);
							continue;
						}
					}

					var observeTop = WSObj.fetchOffsetTop(elementData['observeTop'], scrollTop, clientTop);
					if (scrollTop > observeTop - elementData['observeTopOffset']) {
						WSObj.show(i, 'top', scrollTop, clientHeight);
					} else if (elementData['state'] == 'shown') {
						WSObj.hide(i);
					}
					break;
				case 'bottom':
					clientHeight = document.documentElement.clientHeight;

					if (elementData['observeTop'] != '') {
						var observeTop = WSObj.fetchOffsetTop(elementData['observeTop'], scrollTop, clientTop);
						if (scrollTop + clientHeight - elementData['elementOffsetHeight'] - elementData['observeBottomOffset'] < observeTop) {
							WSObj.hide(i);
							continue;
						}
					}

					var observeBottom = WSObj.fetchOffsetTop(elementData['observeBottom'], scrollTop, clientTop);
					if (scrollTop + clientHeight < observeBottom + WSObj.$(elementData['observeBottom']).offsetHeight + elementData['observeBottomOffset']) {
						WSObj.show(i, 'bottom', scrollTop, clientHeight);
					} else if (elementData['state'] == 'shown') {
						WSObj.hide(i);
					}
					break;
				default:
					clientHeight = document.documentElement.clientHeight;

					var observeTop = WSObj.fetchOffsetTop(elementData['observeTop'], scrollTop, clientTop);
					if (scrollTop + clientHeight - elementData['elementOffsetHeight'] - elementData['observeBottomOffset'] < observeTop) {
						WSObj.hide(i);
						continue;
					}

					var observeBottom = WSObj.fetchOffsetTop(elementData['observeBottom'], scrollTop, clientTop);
					var observeBottomOffsetHeight = WSObj.$(elementData['observeBottom']).offsetHeight;
					if (scrollTop + elementData['elementOffsetHeight'] + elementData['observeTopOffset'] > observeBottom + observeBottomOffsetHeight) {
						WSObj.hide(i);
						continue;
					}

					if (scrollTop > observeTop - elementData['observeTopOffset']) {
						WSObj.show(i, 'top', scrollTop, clientHeight);
						break;
					}

					if (scrollTop + clientHeight < observeBottom + observeBottomOffsetHeight + elementData['observeBottomOffset']) {
						WSObj.show(i, 'bottom', scrollTop, clientHeight);
					} else if (elementData['state'] == 'shown') {
						WSObj.hide(i);
					}

					break;
			}
		}
	}

	WSObj.show = function(indexKey, type, scrollTop, clientHeight) {
		WSObj.objData[indexKey]['state'] = 'shown';
		WSObj.objData[indexKey]['element'].style.display = '';
		if (type == 'top') {
			if (!WSObj.isIE || WSObj.isIE >= 7) {
				WSObj.objData[indexKey]['element'].style.bottom = '';
				WSObj.objData[indexKey]['element'].style.top = WSObj.objData[indexKey]['observeTopOffset'] + 'px';
			} else {
				WSObj.objData[indexKey]['element'].style.top = (scrollTop + WSObj.objData[indexKey]['observeTopOffset']) + 'px';
			}
		} else {
			if (!WSObj.isIE || WSObj.isIE >= 7) {
				WSObj.objData[indexKey]['element'].style.top = '';
				WSObj.objData[indexKey]['element'].style.bottom = WSObj.objData[indexKey]['observeBottomOffset'] + 'px';
			} else {
				WSObj.objData[indexKey]['element'].style.top = (scrollTop + clientHeight - WSObj.objData[indexKey]['elementOffsetHeight'] - WSObj.objData[indexKey]['observeBottomOffset']) + 'px';
			}
		}
	}

	WSObj.hide = function(indexKey) {
		WSObj.objData[indexKey]['state'] = 'hide';
		WSObj.objData[indexKey]['element'].style.display = 'none';
	}

	WSObj.fetchOffsetTop = function(objId, scrollTop, clientTop) {
		var obj = WSObj.$(objId);
		var offsetTop = 0;

		if (obj.getBoundingClientRect){
			var rect = obj.getBoundingClientRect();
			offsetTop = rect.top + scrollTop - clientTop;
		}
		if (offsetTop <= 0) {
			offsetTop = obj.offsetTop;
			while ((obj = obj.offsetParent) != null) {
				offsetTop += obj.offsetTop;
			}
		}

		return offsetTop;
	}

	WSObj.fetchOffsetLeft = function(objId, scrollLeft, clientLeft) {
		var obj = WSObj.$(objId);
		var offsetLeft = 0;

		if (obj.getBoundingClientRect){
			var rect = obj.getBoundingClientRect();
			offsetLeft = rect.left + scrollLeft - clientLeft;
		}
		if (offsetLeft <= 0) {
			offsetLeft = obj.offsetLeft;
			while((obj = obj.offsetParent) != null) {
				offsetLeft += obj.offsetLeft;
			}
		}

		return offsetLeft;
	}

	WSObj.$ = function (obj) {
		return typeof obj == 'string' ? document.getElementById(obj) : obj;
	}

	return WSObj;
}



// last revised: 13:29 9/8/2010
// ModuleElementMove should be loaded before using (for moveUp and moveDown)
// in IE6, some changes done may be lost (e.g. click the checkbox) after moveUp/moveDown/drag
// tested on fx3, fx3.5, fx3.6, ie6, ie7, ie8, ie9b4, chrome5, opera10, safari5, XHTML 1.0 Transitional

// input orderData : array('objId1' => 1, 'objId10' => 2, 'objId9' => 3)
// the order will be stored and updated in WSObj.orderData
function ModuleListOrder(container, elementPrefix, controlElementPrefix, orderData) {
    var WSObj = new Object();

    var userAgent = navigator.userAgent.toLowerCase();
    WSObj.isIE = window.ActiveXObject && userAgent.indexOf('msie') != -1 && userAgent.substr(userAgent.indexOf('msie') + 5, 3);

    // initialization
    WSObj.container = typeof container == 'string' ? document.getElementById(container) : container;
    WSObj.elementPrefix = elementPrefix;
    WSObj.controlElementPrefix = controlElementPrefix;
    WSObj.orderData = orderData;
	
	WSObj.changeStatusByDrag = function(objId){}; // this function will init by external

    var sortable = new Array();
    for (var id in WSObj.orderData) {
        sortable.push([id, WSObj.orderData[id]]);
    }
    sortable.sort(function(a, b) {return a[1] - b[1]});

    WSObj.initialPossibleValue = [];
    for (var i in sortable) {
        WSObj.initialPossibleValue.push(sortable[i][1]);
    }


    // drag initialization of all items
    WSObj.dragInit = function (id) {
        var mousedownEvent = typeof document.getElementById(WSObj.controlElementPrefix + id).onmousedown != 'undefined' && document.getElementById(WSObj.controlElementPrefix + id).onmousedown != null ? document.getElementById(WSObj.controlElementPrefix + id).onmousedown : function () {};
        document.getElementById(WSObj.controlElementPrefix + id).onmousedown = function (e) {
            mousedownEvent();
            WSObj.drag(id, e, 1);
        };
    }
    for (var i in WSObj.orderData) {
        WSObj.dragInit(i);
    }

    WSObj.elementMove = new ModuleElementMove();

    WSObj.getOrderData = function () {
        WSObj.updateDataSequence();

        var resultArray = {};
        for (var order in WSObj.dataSequence) {
            resultArray[WSObj.dataSequence[order][0]] = WSObj.initialPossibleValue[order];
        }

        return resultArray;
    }

    // e.g. array(0 => array(objId1, offsetHeight1, accumulatedHeight1), 1 => array(objId2, offsetHeight2, accumulatedHeight2), 3 => array(objId3, offsetHeight3, accumulatedHeight3))
    WSObj.dataSequence = {};
    WSObj.updateDataSequence = function (getOffsetHeight) {
        var sortable = new Array();
        for (var id in WSObj.orderData) {
            sortable.push([id, WSObj.orderData[id]]);
        }
        sortable.sort(function(a, b) {return a[1] - b[1]});

        var offsetAccumulated = 0;
        var count = 0;
        var dataSequence = new Object();
        for (var i in sortable) {
        	if (getOffsetHeight) {
            	dataSequence[count] = [sortable[i][0], WSObj.$(WSObj.elementPrefix + sortable[i][0]).offsetHeight];
            } else {
            	dataSequence[count] = [sortable[i][0], 0];
            }
            WSObj.orderData[sortable[i][0]] = count;
            offsetAccumulated += dataSequence[count][1];
            dataSequence[count][2] = offsetAccumulated - (dataSequence[count][1] / 2);
            count++;
        }

        WSObj.dataSequence = dataSequence;
    }

    // e.g. array(clientX, clientY, initial left, initial top)
    WSObj.dragStartInfo = new Array();
    // record the current position
    WSObj.currentPos = -1;
    // an indicator to indicate whether the shadow object and preview/dummy (empty) object is created or not
    WSObj.dragInitialized = 0;
    // preserve the drag object and replace it back after drag end
    WSObj.preserveContent = '';
    // preserve the documentElement event and recover it after the drag
    WSObj.documentElementEventPreserve = [];
    // record the offsetTop of first element of the container, if the border/padding of the container is not zero, it is useful to correct the pixel shifted
    WSObj.offsetTopOfContainer = false;
    // record the movable range of the mouse (vertical)
    WSObj.containerOffsetLimit = [0,0];
	
	WSObj.beforeDragSequenceArray = new Array();
	
    WSObj.drag = function(objId, e, op)
    {
        var mainObj = WSObj.$(WSObj.elementPrefix + objId);
        if (op == 1) {
			WSObj.beforeDragSequenceArray = WSObj.getOrderData();
		
            WSObj.selectable(document.documentElement, false);

            WSObj.documentElementEventPreserve['onmousemove'] = WSObj.isDefined(document.documentElement.onmousemove) ? document.documentElement.onmousemove : function () {};
            document.documentElement.onmousemove = function(event) {
                WSObj.drag(objId, event, 2);
            };
            WSObj.documentElementEventPreserve['onmouseup'] = WSObj.isDefined(document.documentElement.onmouseup) ? document.documentElement.onmouseup : function () {};
            document.documentElement.onmouseup = function(event) {
                WSObj.drag(objId, event, 3);
            };

            var offset = WSObj.fetchOffset(mainObj.id);
            WSObj.dragStartInfo = WSObj.isIE ? [event.clientX, event.clientY] : [e.clientX, e.clientY];
            WSObj.dragStartInfo[2] = parseInt(offset['left']);
            WSObj.dragStartInfo[3] = parseInt(offset['top']);

            WSObj.updateDataSequence(true);

            for (var order in WSObj.dataSequence) {
                if (WSObj.dataSequence[order][0] == objId) {
                    WSObj.currentPos = order;
                }
            }

            WSObj.dragInitialized = 0;
        } else if (op == 2) {
            if (WSObj.dragInitialized == 0) {
                WSObj.dragInitialized = 1;

                var containerOffsetTemp = WSObj.fetchOffset(WSObj.container.id);
                var firstElementOffsetTemp = WSObj.fetchOffset(WSObj.elementPrefix + WSObj.dataSequence[0][0]);
                if (WSObj.offsetTopOfContainer == false) {
                    WSObj.offsetTopOfContainer = firstElementOffsetTemp['top'] - containerOffsetTemp['top'];
                }

                WSObj.preserveContent = mainObj;

                var shadowObj = mainObj.cloneNode(true);
                shadowObj.id = 'shadowObj_' + objId;
                shadowObj.style.position = 'absolute';
                shadowObj.style.zIndex = 99;
                var shadowObjOffset = WSObj.fetchOffset(mainObj.id);
                shadowObj.style.top = shadowObjOffset['top'] + 'px';
                shadowObj.style.left = shadowObjOffset['left'] + 'px';

                var dummyObjInner = document.createElement('div');
                dummyObjInner.style.border = "4px dashed #888";
                dummyObjInner.style.height = mainObj.offsetHeight + 'px';

                var dummyObj = document.createElement('div');
                dummyObj.id = mainObj.id;
                dummyObj.style.height = mainObj.offsetHeight + 'px';
                dummyObj.appendChild(dummyObjInner);

                var mainObjHeight = mainObj.offsetHeight;
                WSObj.container.replaceChild(dummyObj, WSObj.$(WSObj.elementPrefix + objId));
                if (dummyObjInner.offsetHeight > mainObjHeight) {
                    dummyObjInner.style.height = (mainObjHeight - (dummyObjInner.offsetHeight - mainObjHeight)) + 'px';
                }

                WSObj.container.appendChild(shadowObj);
                WSObj.setOpacity(shadowObj.id, 0.6);

                var lastElement = '';
                for (var orderTemp in WSObj.dataSequence) {
                    lastElement = WSObj.dataSequence[orderTemp][0];
                }
                var offsetTemp = WSObj.fetchOffset(WSObj.elementPrefix + lastElement);
                WSObj.containerOffsetLimit[1] = offsetTemp['top'] + WSObj.$(WSObj.elementPrefix + lastElement).offsetHeight - WSObj.$(WSObj.elementPrefix + objId).offsetHeight;
            }

            var shadowObj = WSObj.$('shadowObj_' + objId);
            mainObj = WSObj.$(WSObj.elementPrefix + objId);

            var containerOffset = WSObj.fetchOffset(WSObj.container.id);

            WSObj.containerOffsetLimit[0] = containerOffset['top'] + WSObj.offsetTopOfContainer;

            var dragInfo = WSObj.isIE ? [event.clientX, event.clientY] : [e.clientX, e.clientY];
            var top = WSObj.dragStartInfo[3] + (dragInfo[1] - WSObj.dragStartInfo[1]);
            if (top < WSObj.containerOffsetLimit[0]) top = WSObj.containerOffsetLimit[0];
            if (top > WSObj.containerOffsetLimit[1]) top = WSObj.containerOffsetLimit[1];

            shadowObj.style.top = top + 'px';

            var finalPosition = -1, finalPosition_top = -1, finalPosition_bottom = -1;
            var tempValue, bottomOffset = top + mainObj.offsetHeight;
            for (var order in WSObj.dataSequence) {
                if (order == WSObj.currentPos) continue;
                tempValue = WSObj.containerOffsetLimit[0] + WSObj.dataSequence[order][2];
                if (order < WSObj.currentPos && top > tempValue) {
                    finalPosition_top = parseInt(order) + 1;
                }
                if (order > WSObj.currentPos && bottomOffset > tempValue) {
                    finalPosition_bottom = order;
                }
            }
            finalPosition = finalPosition_top != -1 && finalPosition_top != WSObj.currentPos ? finalPosition_top : finalPosition_bottom;
            if (top < WSObj.containerOffsetLimit[0] + WSObj.dataSequence[0][2] && finalPosition != parseInt(WSObj.currentPos) + 1) {
            	finalPosition = 0;
            }
            if (finalPosition != -1 && WSObj.currentPos != finalPosition) {
                WSObj.moveToPosition(objId, finalPosition, function() { WSObj.updateDataSequence(true); });
                WSObj.currentPos = finalPosition;
            }
        } else if (op == 3) {
            if (WSObj.dragInitialized) WSObj.container.removeChild(WSObj.$('shadowObj_' + objId));
            if (WSObj.dragInitialized) WSObj.container.replaceChild(WSObj.preserveContent, WSObj.$(WSObj.elementPrefix + objId));
			
			// get current
			arr = new Array();
			arr = WSObj.getOrderData();
			
			tmpcount = 1; // since it start with 1
			for (var o_order in WSObj.beforeDragSequenceArray) {
				if (o_order != arr[tmpcount]){
					if (WSObj.isDefined(WSObj.changeStatusByDrag)) WSObj.changeStatusByDrag(o_order);
				}
				tmpcount++;
			}
			
			
			
			WSObj.selectable(document.documentElement, true);
            document.documentElement.onmousemove = WSObj.documentElementEventPreserve['onmousemove'];
            document.documentElement.onmouseup = WSObj.documentElementEventPreserve['onmouseup'];
        }
    }

    WSObj.moveToPosition = function(objId, moveTo, callbackFunc) {
        WSObj.updateDataSequence(false);

        var objPosition = -1;
        var targetObj = ['', ''];
        var objAfterTargetObj = ['', ''];

        var id, dataNum = 0;
        for (var order in WSObj.dataSequence) {
            id = WSObj.dataSequence[order][0];
            if (id == objId) {
                objPosition = order;
                if (moveTo == objPosition) return;
            }
            if (targetObj[1] != '' && objAfterTargetObj[1] == '') {
                objAfterTargetObj[0] = order;
                objAfterTargetObj[1] = id;
            }
            if (order == moveTo) {
                targetObj[0] = order;
                targetObj[1] = id;
            }
            dataNum++;
        }

        WSObj.orderData[objId] = WSObj.orderData[targetObj[1]];

        var start = false;
        if (objPosition > targetObj[0]) {
            for (var order in WSObj.dataSequence) {
                if (order == objPosition) break;
                if (order == targetObj[0]) start = true;
                if (start) WSObj.orderData[WSObj.dataSequence[order][0]]++;
            }
        } else {
            for (var order in WSObj.dataSequence) {
                if (start) WSObj.orderData[WSObj.dataSequence[order][0]]--;
                if (order == targetObj[0]) break;
                if (order == objPosition) start = true;
            }
        }

        var preserveObj = WSObj.$(WSObj.elementPrefix + objId);
        if (moveTo == dataNum - 1) {
            WSObj.container.removeChild(WSObj.$(WSObj.elementPrefix + objId));
            WSObj.container.appendChild(preserveObj);
        } else {
            if (objPosition < targetObj[0]) {
                WSObj.container.removeChild(WSObj.$(WSObj.elementPrefix + objId));
                WSObj.container.insertBefore(preserveObj, WSObj.$(WSObj.elementPrefix + objAfterTargetObj[1]));
            } else {
                WSObj.container.removeChild(WSObj.$(WSObj.elementPrefix + objId));
                WSObj.container.insertBefore(preserveObj, WSObj.$(WSObj.elementPrefix + targetObj[1]));
            }
        }

        if (WSObj.isDefined(callbackFunc)) callbackFunc();
		
    }

    WSObj.moveUp = function(objId) {
        WSObj.updateDataSequence(false);

        var objectJustBeforeMe = ['',''];

        var objPosition = -1;
        for (var i in WSObj.dataSequence) {
            if (WSObj.dataSequence[i][0] == objId) {
                objPosition = i;
                break;
            } else if (objPosition == -1) {
                objectJustBeforeMe = [i, WSObj.dataSequence[i][0]];
            }
        }

        var preserveObj = WSObj.$(WSObj.elementPrefix + objId);
        if (objectJustBeforeMe[1] != '') {
            var getBeforeObjOffset = WSObj.fetchOffset(WSObj.elementPrefix + objectJustBeforeMe[1]);

            WSObj.elementMove.illutionMove(WSObj.elementPrefix + objId, 1, 99, function () { }, getBeforeObjOffset['top']);
            WSObj.elementMove.illutionMove(WSObj.elementPrefix + objectJustBeforeMe[1], 0.4, 98, function () { }, getBeforeObjOffset['top'] + WSObj.$(WSObj.elementPrefix + objId).offsetHeight);

            WSObj.container.removeChild(WSObj.$(WSObj.elementPrefix + objId));
            WSObj.container.insertBefore(preserveObj, WSObj.$(WSObj.elementPrefix + objectJustBeforeMe[1]));

            WSObj.orderData[objId] = objectJustBeforeMe[0];
            WSObj.orderData[objectJustBeforeMe[1]] = objPosition;
        }
    }

    WSObj.moveDown = function(objId) {
        WSObj.updateDataSequence(false);

        var objectFutureBeforeMe = ['',''];
        var objectJustAfterMe = ['',''];

        var objPosition = -1;
        for (var i in WSObj.dataSequence) {
            if (objPosition != -1) {
                objectJustAfterMe = [i, WSObj.dataSequence[i][0]];
                i = parseInt(i);
                if (typeof WSObj.dataSequence[(i+1)] != 'undefined') {
                    objectFutureBeforeMe = [(i+1), WSObj.dataSequence[(i+1)][0]];
                }
                break;
            }
            if (WSObj.dataSequence[i][0] == objId) {
                objPosition = i;
            }
        }

        var determine = (objectJustAfterMe[1] != '' && objectFutureBeforeMe[1] != '') || (objectJustAfterMe[1] != '' && objectFutureBeforeMe[1] == '');
        if (determine) {
            var getObjOffset = WSObj.fetchOffset(WSObj.elementPrefix + objId);

            WSObj.elementMove.illutionMove(WSObj.elementPrefix + objId, 1, 99, function () { }, getObjOffset['top'] + WSObj.$(WSObj.elementPrefix + objectJustAfterMe[1]).offsetHeight);
            WSObj.elementMove.illutionMove(WSObj.elementPrefix + objectJustAfterMe[1], 0.4, 98, function () { }, getObjOffset['top']);

            var preserveObj = WSObj.$(WSObj.elementPrefix + objId);
            WSObj.container.removeChild(WSObj.$(WSObj.elementPrefix + objId));
        }

        if (objectJustAfterMe[1] != '' && objectFutureBeforeMe[1] != '') {
            WSObj.container.insertBefore(preserveObj, WSObj.$(WSObj.elementPrefix + objectFutureBeforeMe[1]));
        } else if (objectJustAfterMe[1] != '' && objectFutureBeforeMe[1] == '') {
            WSObj.container.appendChild(preserveObj);
        }

        if (determine) {
            WSObj.orderData[objId] = objectJustAfterMe[0];
            WSObj.orderData[objectJustAfterMe[1]] = objPosition;
        }
    }

    WSObj.selectable = function (obj, state) {
        obj = WSObj.$(obj);
        // make the main object not selectable
        if (obj.style.MozUserSelect !== undefined) {
            obj.style.MozUserSelect = state ? 'all' : 'none';
        } else if (obj.style.webkitUserSelect !== undefined) {
            obj.style.webkitUserSelect = state ? 'auto' : 'none';
        } else {
            obj.unselectable = state ? 'off' : 'on';
            obj.onselectstart = state ? function () { } : function () { return false; };
        }
    }

    WSObj.fetchOffset = function(objId) {
        var obj = WSObj.$(objId);
		var offsetLeft = 0, offsetTop = 0;

		if(obj.getBoundingClientRect){
			var rect = obj.getBoundingClientRect();
			var scrollTop = Math.max(document.documentElement.scrollTop, document.body.scrollTop);
			var scrollLeft = Math.max(document.documentElement.scrollLeft, document.body.scrollLeft);
			if(document.documentElement.dir == 'rtl') {
				scrollLeft = scrollLeft + document.documentElement.clientWidth - document.documentElement.scrollWidth;
			}
			offsetLeft = rect.left + scrollLeft - document.documentElement.clientLeft;
			offsetTop = rect.top + scrollTop - document.documentElement.clientTop;
		}
		if(offsetLeft <= 0 || offsetTop <= 0) {
			offsetLeft = obj.offsetLeft;
			offsetTop = obj.offsetTop;
			while((obj = obj.offsetParent) != null) {
				offsetLeft += obj.offsetLeft;
				offsetTop += obj.offsetTop;
			}
		}

		return { 'left' : offsetLeft, 'top' : offsetTop };
    }

    WSObj.doane = function (event) {
        e = event ? event : window.event;
        if (WSObj.isIE) {
            e.cancelBubble = true;
        } else {
            e.stopPropagation();
        }
    }

    WSObj.setOpacity = function(obj, opacityValue) {
        obj = WSObj.$(obj);
        if (WSObj.isIE && WSObj.isIE <= 8) {
            obj.style.filter = 'alpha(opacity=' + (opacityValue * 100) + ')';
        } else {
            obj.style.opacity = opacityValue;
        }
    }

    WSObj.$ = function (obj) {
        return typeof obj == 'string' ? document.getElementById(obj) : obj;
    }

    WSObj.isDefined = function (target) {
        return typeof target != 'undefined' && target != null;
    }

    return WSObj;
}

// added by chris 
// jsut add one (last entry) in List Order
function ModuleAddLastListOrder(old_WSObj, container, elementPrefix, controlElementPrefix, orderData, new_id, new_order) {
    var WSObj = old_WSObj;
	/*
	{
    var userAgent = navigator.userAgent.toLowerCase();
    WSObj.isIE = window.ActiveXObject && userAgent.indexOf('msie') != -1 && userAgent.substr(userAgent.indexOf('msie') + 5, 3);

    // initialization
    WSObj.container = typeof container == 'string' ? document.getElementById(container) : container;
    WSObj.elementPrefix = elementPrefix;
    WSObj.controlElementPrefix = controlElementPrefix;
	}
	*/
    // WSObj.orderData = orderData;
    WSObj.orderData[new_id] = new_order;


    // var sortable = new Array();
    // for (var id in orderData) {
        // sortable.push([id, orderData[id]]);
    // }
    // sortable.sort(function(a, b) {return a[1] - b[1]});

    // WSObj.initialPossibleValue = [];
	// var last_sortable;
    // for (var i in sortable) {
		// last_sortable = sortable[i][1];
        // // WSObj.initialPossibleValue.push(sortable[i][1]);
    // }
	WSObj.initialPossibleValue.push(new_order);
	/*
    // drag initialization of all items
	if(!WSObj.dragInit) { // if this method does not exist..
    WSObj.dragInit = function (id) {
        var mousedownEvent = typeof document.getElementById(WSObj.controlElementPrefix + id).onmousedown != 'undefined' && document.getElementById(WSObj.controlElementPrefix + id).onmousedown != null ? document.getElementById(WSObj.controlElementPrefix + id).onmousedown : function () {};
        document.getElementById(WSObj.controlElementPrefix + id).onmousedown = function (e) {
            mousedownEvent();
            WSObj.drag(id, e, 1);
        };
    }
	}
	*/
	// var last_i;
    // for (var i in orderData) {
		// last_i = i;
        // // WSObj.dragInit(i);
    // }
	WSObj.dragInit(new_id);
	/*
	{
	if(!WSObj.elementMove) { // if this method does not exist..
    WSObj.elementMove = new ModuleElementMove();
	}
	
	if(!WSObj.getOrderData) { // if this method does not exist..
    WSObj.getOrderData = function () {
        WSObj.updateDataSequence();

        var resultArray = {};
        for (var order in WSObj.dataSequence) {
            resultArray[WSObj.dataSequence[order][0]] = WSObj.initialPossibleValue[order];
        }

        return resultArray;
    }
	}

    // e.g. array(0 => array(objId1, offsetHeight1, accumulatedHeight1), 1 => array(objId2, offsetHeight2, accumulatedHeight2), 3 => array(objId3, offsetHeight3, accumulatedHeight3))
    WSObj.dataSequence = {};
	if(!WSObj.updateDataSequence) { // if this method does not exist..
    WSObj.updateDataSequence = function (getOffsetHeight) {
        var sortable = new Array();
        for (var id in WSObj.orderData) {
            sortable.push([id, WSObj.orderData[id]]);
        }
        sortable.sort(function(a, b) {return a[1] - b[1]});

        var offsetAccumulated = 0;
        var count = 0;
        var dataSequence = new Object();
        for (var i in sortable) {
        	if (getOffsetHeight) {
            	dataSequence[count] = [sortable[i][0], WSObj.$(WSObj.elementPrefix + sortable[i][0]).offsetHeight];
            } else {
            	dataSequence[count] = [sortable[i][0], 0];
            }
            WSObj.orderData[sortable[i][0]] = count;
            offsetAccumulated += dataSequence[count][1];
            dataSequence[count][2] = offsetAccumulated - (dataSequence[count][1] / 2);
            count++;
        }

        WSObj.dataSequence = dataSequence;
    }
	}
	*/
    // e.g. array(clientX, clientY, initial left, initial top)
    WSObj.dragStartInfo = new Array();
    // record the current position
    WSObj.currentPos = -1;
    // an indicator to indicate whether the shadow object and preview/dummy (empty) object is created or not
    WSObj.dragInitialized = 0;
    // preserve the drag object and replace it back after drag end
    WSObj.preserveContent = '';
    // preserve the documentElement event and recover it after the drag
    WSObj.documentElementEventPreserve = [];
    // record the offsetTop of first element of the container, if the border/padding of the container is not zero, it is useful to correct the pixel shifted
    WSObj.offsetTopOfContainer = false;
    // record the movable range of the mouse (vertical)
    WSObj.containerOffsetLimit = [0,0];
	/*
	if(!WSObj.drag) { // if this method does not exist..
    WSObj.drag = function(objId, e, op)
    {
        var mainObj = WSObj.$(WSObj.elementPrefix + objId);
        if (op == 1) {
            WSObj.selectable(document.documentElement, false);

            WSObj.documentElementEventPreserve['onmousemove'] = WSObj.isDefined(document.documentElement.onmousemove) ? document.documentElement.onmousemove : function () {};
            document.documentElement.onmousemove = function(event) {
                WSObj.drag(objId, event, 2);
            };
            WSObj.documentElementEventPreserve['onmouseup'] = WSObj.isDefined(document.documentElement.onmouseup) ? document.documentElement.onmouseup : function () {};
            document.documentElement.onmouseup = function(event) {
                WSObj.drag(objId, event, 3);
            };

            var offset = WSObj.fetchOffset(mainObj.id);
            WSObj.dragStartInfo = WSObj.isIE ? [event.clientX, event.clientY] : [e.clientX, e.clientY];
            WSObj.dragStartInfo[2] = parseInt(offset['left']);
            WSObj.dragStartInfo[3] = parseInt(offset['top']);

            WSObj.updateDataSequence(true);

            for (var order in WSObj.dataSequence) {
                if (WSObj.dataSequence[order][0] == objId) {
                    WSObj.currentPos = order;
                }
            }

            WSObj.dragInitialized = 0;
        } else if (op == 2) {
            if (WSObj.dragInitialized == 0) {
                WSObj.dragInitialized = 1;

                var containerOffsetTemp = WSObj.fetchOffset(WSObj.container.id);
                var firstElementOffsetTemp = WSObj.fetchOffset(WSObj.elementPrefix + WSObj.dataSequence[0][0]);
                if (WSObj.offsetTopOfContainer == false) {
                    WSObj.offsetTopOfContainer = firstElementOffsetTemp['top'] - containerOffsetTemp['top'];
                }

                WSObj.preserveContent = mainObj;

                var shadowObj = mainObj.cloneNode(true);
                shadowObj.id = 'shadowObj_' + objId;
                shadowObj.style.position = 'absolute';
                shadowObj.style.zIndex = 99;
                var shadowObjOffset = WSObj.fetchOffset(mainObj.id);
                shadowObj.style.top = shadowObjOffset['top'] + 'px';
                shadowObj.style.left = shadowObjOffset['left'] + 'px';

                var dummyObjInner = document.createElement('div');
                dummyObjInner.style.border = "4px dashed #888";
                dummyObjInner.style.height = mainObj.offsetHeight + 'px';

                var dummyObj = document.createElement('div');
                dummyObj.id = mainObj.id;
                dummyObj.style.height = mainObj.offsetHeight + 'px';
                dummyObj.appendChild(dummyObjInner);

                var mainObjHeight = mainObj.offsetHeight;
                WSObj.container.replaceChild(dummyObj, WSObj.$(WSObj.elementPrefix + objId));
                if (dummyObjInner.offsetHeight > mainObjHeight) {
                    dummyObjInner.style.height = (mainObjHeight - (dummyObjInner.offsetHeight - mainObjHeight)) + 'px';
                }

                WSObj.container.appendChild(shadowObj);
                WSObj.setOpacity(shadowObj.id, 0.6);

                var lastElement = '';
                for (var orderTemp in WSObj.dataSequence) {
                    lastElement = WSObj.dataSequence[orderTemp][0];
                }
                var offsetTemp = WSObj.fetchOffset(WSObj.elementPrefix + lastElement);
                WSObj.containerOffsetLimit[1] = offsetTemp['top'] + WSObj.$(WSObj.elementPrefix + lastElement).offsetHeight - WSObj.$(WSObj.elementPrefix + objId).offsetHeight;
            }

            var shadowObj = WSObj.$('shadowObj_' + objId);
            mainObj = WSObj.$(WSObj.elementPrefix + objId);

            var containerOffset = WSObj.fetchOffset(WSObj.container.id);

            WSObj.containerOffsetLimit[0] = containerOffset['top'] + WSObj.offsetTopOfContainer;

            var dragInfo = WSObj.isIE ? [event.clientX, event.clientY] : [e.clientX, e.clientY];
            var top = WSObj.dragStartInfo[3] + (dragInfo[1] - WSObj.dragStartInfo[1]);
            if (top < WSObj.containerOffsetLimit[0]) top = WSObj.containerOffsetLimit[0];
            if (top > WSObj.containerOffsetLimit[1]) top = WSObj.containerOffsetLimit[1];

            shadowObj.style.top = top + 'px';

            var finalPosition = -1, finalPosition_top = -1, finalPosition_bottom = -1;
            var tempValue, bottomOffset = top + mainObj.offsetHeight;
            for (var order in WSObj.dataSequence) {
                if (order == WSObj.currentPos) continue;
                tempValue = WSObj.containerOffsetLimit[0] + WSObj.dataSequence[order][2];
                if (order < WSObj.currentPos && top > tempValue) {
                    finalPosition_top = parseInt(order) + 1;
                }
                if (order > WSObj.currentPos && bottomOffset > tempValue) {
                    finalPosition_bottom = order;
                }
            }
            finalPosition = finalPosition_top != -1 && finalPosition_top != WSObj.currentPos ? finalPosition_top : finalPosition_bottom;
            if (top < WSObj.containerOffsetLimit[0] + WSObj.dataSequence[0][2] && finalPosition != parseInt(WSObj.currentPos) + 1) {
            	finalPosition = 0;
            }
            if (finalPosition != -1 && WSObj.currentPos != finalPosition) {
                WSObj.moveToPosition(objId, finalPosition, function() { WSObj.updateDataSequence(true); });
                WSObj.currentPos = finalPosition;
            }
        } else if (op == 3) {
            if (WSObj.dragInitialized) WSObj.container.removeChild(WSObj.$('shadowObj_' + objId));
            if (WSObj.dragInitialized) WSObj.container.replaceChild(WSObj.preserveContent, WSObj.$(WSObj.elementPrefix + objId));
            WSObj.selectable(document.documentElement, true);
            document.documentElement.onmousemove = WSObj.documentElementEventPreserve['onmousemove'];
            document.documentElement.onmouseup = WSObj.documentElementEventPreserve['onmouseup'];
        }
    }
	}
	if(!WSObj.moveToPosition) { // if this method does not exist..
    WSObj.moveToPosition = function(objId, moveTo, callbackFunc) {
        WSObj.updateDataSequence(false);

        var objPosition = -1;
        var targetObj = ['', ''];
        var objAfterTargetObj = ['', ''];

        var id, dataNum = 0;
        for (var order in WSObj.dataSequence) {
            id = WSObj.dataSequence[order][0];
            if (id == objId) {
                objPosition = order;
                if (moveTo == objPosition) return;
            }
            if (targetObj[1] != '' && objAfterTargetObj[1] == '') {
                objAfterTargetObj[0] = order;
                objAfterTargetObj[1] = id;
            }
            if (order == moveTo) {
                targetObj[0] = order;
                targetObj[1] = id;
            }
            dataNum++;
        }

        WSObj.orderData[objId] = WSObj.orderData[targetObj[1]];

        var start = false;
        if (objPosition > targetObj[0]) {
            for (var order in WSObj.dataSequence) {
                if (order == objPosition) break;
                if (order == targetObj[0]) start = true;
                if (start) WSObj.orderData[WSObj.dataSequence[order][0]]++;
            }
        } else {
            for (var order in WSObj.dataSequence) {
                if (start) WSObj.orderData[WSObj.dataSequence[order][0]]--;
                if (order == targetObj[0]) break;
                if (order == objPosition) start = true;
            }
        }

        var preserveObj = WSObj.$(WSObj.elementPrefix + objId);
        if (moveTo == dataNum - 1) {
            WSObj.container.removeChild(WSObj.$(WSObj.elementPrefix + objId));
            WSObj.container.appendChild(preserveObj);
        } else {
            if (objPosition < targetObj[0]) {
                WSObj.container.removeChild(WSObj.$(WSObj.elementPrefix + objId));
                WSObj.container.insertBefore(preserveObj, WSObj.$(WSObj.elementPrefix + objAfterTargetObj[1]));
            } else {
                WSObj.container.removeChild(WSObj.$(WSObj.elementPrefix + objId));
                WSObj.container.insertBefore(preserveObj, WSObj.$(WSObj.elementPrefix + targetObj[1]));
            }
        }

        if (WSObj.isDefined(callbackFunc)) callbackFunc();
    }
	}
	if(!WSObj.moveUp) { // if this method does not exist..
    WSObj.moveUp = function(objId) {
        WSObj.updateDataSequence(false);

        var objectJustBeforeMe = ['',''];

        var objPosition = -1;
        for (var i in WSObj.dataSequence) {
            if (WSObj.dataSequence[i][0] == objId) {
                objPosition = i;
                break;
            } else if (objPosition == -1) {
                objectJustBeforeMe = [i, WSObj.dataSequence[i][0]];
            }
        }

        var preserveObj = WSObj.$(WSObj.elementPrefix + objId);
        if (objectJustBeforeMe[1] != '') {
            var getBeforeObjOffset = WSObj.fetchOffset(WSObj.elementPrefix + objectJustBeforeMe[1]);

            WSObj.elementMove.illutionMove(WSObj.elementPrefix + objId, 1, 99, function () { }, getBeforeObjOffset['top']);
            WSObj.elementMove.illutionMove(WSObj.elementPrefix + objectJustBeforeMe[1], 0.4, 98, function () { }, getBeforeObjOffset['top'] + WSObj.$(WSObj.elementPrefix + objId).offsetHeight);

            WSObj.container.removeChild(WSObj.$(WSObj.elementPrefix + objId));
            WSObj.container.insertBefore(preserveObj, WSObj.$(WSObj.elementPrefix + objectJustBeforeMe[1]));

            WSObj.orderData[objId] = objectJustBeforeMe[0];
            WSObj.orderData[objectJustBeforeMe[1]] = objPosition;
        }
    }
	}
	if(!WSObj.moveDown) { // if this method does not exist..
    WSObj.moveDown = function(objId) {
        WSObj.updateDataSequence(false);

        var objectFutureBeforeMe = ['',''];
        var objectJustAfterMe = ['',''];

        var objPosition = -1;
        for (var i in WSObj.dataSequence) {
            if (objPosition != -1) {
                objectJustAfterMe = [i, WSObj.dataSequence[i][0]];
                i = parseInt(i);
                if (typeof WSObj.dataSequence[(i+1)] != 'undefined') {
                    objectFutureBeforeMe = [(i+1), WSObj.dataSequence[(i+1)][0]];
                }
                break;
            }
            if (WSObj.dataSequence[i][0] == objId) {
                objPosition = i;
            }
        }

        var determine = (objectJustAfterMe[1] != '' && objectFutureBeforeMe[1] != '') || (objectJustAfterMe[1] != '' && objectFutureBeforeMe[1] == '');
        if (determine) {
            var getObjOffset = WSObj.fetchOffset(WSObj.elementPrefix + objId);

            WSObj.elementMove.illutionMove(WSObj.elementPrefix + objId, 1, 99, function () { }, getObjOffset['top'] + WSObj.$(WSObj.elementPrefix + objectJustAfterMe[1]).offsetHeight);
            WSObj.elementMove.illutionMove(WSObj.elementPrefix + objectJustAfterMe[1], 0.4, 98, function () { }, getObjOffset['top']);

            var preserveObj = WSObj.$(WSObj.elementPrefix + objId);
            WSObj.container.removeChild(WSObj.$(WSObj.elementPrefix + objId));
        }

        if (objectJustAfterMe[1] != '' && objectFutureBeforeMe[1] != '') {
            WSObj.container.insertBefore(preserveObj, WSObj.$(WSObj.elementPrefix + objectFutureBeforeMe[1]));
        } else if (objectJustAfterMe[1] != '' && objectFutureBeforeMe[1] == '') {
            WSObj.container.appendChild(preserveObj);
        }

        if (determine) {
            WSObj.orderData[objId] = objectJustAfterMe[0];
            WSObj.orderData[objectJustAfterMe[1]] = objPosition;
        }
    }
	}
	if(!WSObj.selectable) { // if this method does not exist..
    WSObj.selectable = function (obj, state) {
        obj = WSObj.$(obj);
        // make the main object not selectable
        if (obj.style.MozUserSelect !== undefined) {
            obj.style.MozUserSelect = state ? 'all' : 'none';
        } else if (obj.style.webkitUserSelect !== undefined) {
            obj.style.webkitUserSelect = state ? 'auto' : 'none';
        } else {
            obj.unselectable = state ? 'off' : 'on';
            obj.onselectstart = state ? function () { } : function () { return false; };
        }
    }
	}
	if(!WSObj.fetchOffset) { // if this method does not exist..
    WSObj.fetchOffset = function(objId) {
        var obj = WSObj.$(objId);
		var offsetLeft = 0, offsetTop = 0;

		if(obj.getBoundingClientRect){
			var rect = obj.getBoundingClientRect();
			var scrollTop = Math.max(document.documentElement.scrollTop, document.body.scrollTop);
			var scrollLeft = Math.max(document.documentElement.scrollLeft, document.body.scrollLeft);
			if(document.documentElement.dir == 'rtl') {
				scrollLeft = scrollLeft + document.documentElement.clientWidth - document.documentElement.scrollWidth;
			}
			offsetLeft = rect.left + scrollLeft - document.documentElement.clientLeft;
			offsetTop = rect.top + scrollTop - document.documentElement.clientTop;
		}
		if(offsetLeft <= 0 || offsetTop <= 0) {
			offsetLeft = obj.offsetLeft;
			offsetTop = obj.offsetTop;
			while((obj = obj.offsetParent) != null) {
				offsetLeft += obj.offsetLeft;
				offsetTop += obj.offsetTop;
			}
		}

		return { 'left' : offsetLeft, 'top' : offsetTop };
    }
	}
	if(!WSObj.doane) { // if this method does not exist..
    WSObj.doane = function (event) {
        e = event ? event : window.event;
        if (WSObj.isIE) {
            e.cancelBubble = true;
        } else {
            e.stopPropagation();
        }
    }
	}
	if(!WSObj.setOpacity) { // if this method does not exist..
    WSObj.setOpacity = function(obj, opacityValue) {
        obj = WSObj.$(obj);
        if (WSObj.isIE && WSObj.isIE <= 8) {
            obj.style.filter = 'alpha(opacity=' + (opacityValue * 100) + ')';
        } else {
            obj.style.opacity = opacityValue;
        }
    }
	}
	if(!WSObj.$) { // if this method does not exist..
    WSObj.$ = function (obj) {
        return typeof obj == 'string' ? document.getElementById(obj) : obj;
    }
	}
	if(!WSObj.isDefined) { // if this method does not exist..
    WSObj.isDefined = function (target) {
        return typeof target != 'undefined' && target != null;
    }
	}
	}
    */
	return WSObj;
}
